@@ -24,10 +24,3 @@ from nucypher.__about__ import (
 __all__ = [
     "__title__", "__summary__", "__version__", "__author__", "__license__", "__copyright__", "__email__", "__url__"
 ]
-
-
-# Set Default Curve #
-#####################
-
-from umbral.config import set_default_curve
-set_default_curve()
