@@ -18,29 +18,28 @@
 
 import random
 import weakref
+from collections import deque
 from collections.abc import KeysView
 from typing import Optional, Dict, Iterable, List, Tuple, NamedTuple, Union, Any
 
-import binascii
-import itertools
 import maya
-from bytestring_splitter import BytestringSplitter
 from eth_typing import ChecksumAddress
 
-from nucypher.crypto.api import keccak_digest
+from nucypher_core import FleetStateChecksum, NodeMetadata
+
 from nucypher.utilities.logging import Logger
 from .nicknames import Nickname
 
 
 class ArchivedFleetState(NamedTuple):
 
-    checksum: ChecksumAddress
+    checksum: str
     nickname: Nickname
     timestamp: maya.MayaDT
     population: int
 
     def to_json(self):
-        return dict(checksum=self.checksum,
+        return dict(checksum=bytes(self.checksum).hex(),
                     nickname=self.nickname.to_json(),
                     timestamp=self.timestamp.rfc2822(),
                     population=self.population)
@@ -74,22 +73,21 @@ class FleetState:
 
     @classmethod
     def new(cls, this_node: Optional['Ursula'] = None) -> 'FleetState':
-        this_node_ref = weakref.ref(this_node) if this_node is not None else None
-        # Using empty checksum so that JSON library is not confused.
-        # Plus, we do need some checksum anyway. It's a legitimate state after all.
-        return cls(checksum=keccak_digest(b"").hex(),
-                   nodes={},
+        this_node_ref = weakref.ref(this_node) if this_node else None
+        # `this_node` might not have its metadata available yet.
+        this_node_metadata = None
+
+        return cls(nodes={},
                    this_node_ref=this_node_ref,
-                   this_node_metadata=None)
+                   this_node_metadata=this_node_metadata)
 
     def __init__(self,
-                 checksum: str,
                  nodes: Dict[ChecksumAddress, 'Ursula'],
                  this_node_ref: Optional[weakref.ReferenceType],
-                 this_node_metadata: Optional[bytes]):
-
-        self.checksum = checksum
-        self.nickname = Nickname.from_seed(checksum, length=1)
+                 this_node_metadata: Optional[NodeMetadata]):
+        self.checksum = FleetStateChecksum(this_node=this_node_metadata,
+                                           other_nodes=[node.metadata() for node in nodes.values()])
+        self.nickname = Nickname.from_seed(bytes(self.checksum), length=1)
         self._nodes = nodes
         self.timestamp = maya.now()
         self._this_node_ref = this_node_ref
@@ -112,7 +110,7 @@ class FleetState:
             if node.checksum_address in nodes_to_remove:
                 continue
             unknown = node.checksum_address not in self._nodes
-            if unknown or bytes(self._nodes[node.checksum_address]) != bytes(node):
+            if unknown or bytes(self._nodes[node.checksum_address].metadata()) != bytes(node.metadata()):
                 nodes_updated.append(node.checksum_address)
 
         nodes_removed = []
@@ -132,7 +130,7 @@ class FleetState:
 
         if self._this_node_ref is not None and not skip_this_node:
             this_node = self._this_node_ref()
-            this_node_metadata = bytes(this_node)
+            this_node_metadata = this_node.metadata()
             this_node_updated = self._this_node_metadata != this_node_metadata
             this_node_list = [this_node]
         else:
@@ -150,21 +148,13 @@ class FleetState:
             nodes_to_add_dict = {node.checksum_address: node for node in nodes_to_add}
             for checksum_address in diff.nodes_updated:
                 new_node = nodes_to_add_dict[checksum_address]
-                new_node.mature()
                 nodes[checksum_address] = new_node
             for checksum_address in diff.nodes_removed:
                 del nodes[checksum_address]
-
-            all_nodes_sorted = sorted(itertools.chain(this_node_list, nodes.values()),
-                                      key=lambda node: node.checksum_address)
-            joined_metadata = b"".join(bytes(node) for node in all_nodes_sorted)
-            checksum = keccak_digest(joined_metadata).hex()
         else:
             nodes = self._nodes
-            checksum = self.checksum
 
-        new_state = FleetState(checksum=checksum,
-                               nodes=nodes,
+        new_state = FleetState(nodes=nodes,
                                this_node_ref=self._this_node_ref,
                                this_node_metadata=this_node_metadata)
 
@@ -196,22 +186,6 @@ class FleetState:
     def __len__(self):
         return len(self._nodes)
 
-    # TODO: we only send it along with `FLEET_STATES_MATCH`, so it is essentially useless.
-    # But it's hard to change now because older nodes will be looking for it.
-    def snapshot(self) -> bytes:
-        checksum_bytes = binascii.unhexlify(self.checksum)
-        timestamp_bytes = self.timestamp.epoch.to_bytes(4, byteorder="big")
-        return checksum_bytes + timestamp_bytes
-
-    snapshot_splitter = BytestringSplitter(32, 4)
-
-    @staticmethod
-    def unpack_snapshot(data) -> Tuple[str, maya.MayaDT, bytes]:
-        checksum_bytes, timestamp_bytes, remainder = FleetState.snapshot_splitter(data, return_remainder=True)
-        checksum = checksum_bytes.hex()
-        timestamp = maya.MayaDT(int.from_bytes(timestamp_bytes, byteorder="big"))
-        return checksum, timestamp, remainder
-
     def shuffled(self) -> List['Ursula']:
         nodes_we_know_about = list(self._nodes.values())
         random.shuffle(nodes_we_know_about)
@@ -234,7 +208,7 @@ class FleetState:
     def __str__(self):
         return '{checksum} ⇀{nickname}↽ {icon} '.format(icon=self.nickname.icon,
                                                         nickname=self.nickname,
-                                                        checksum=self.checksum[:7])
+                                                        checksum=bytes(self.checksum).hex()[:7])
 
     def __repr__(self):
         return f"FleetState({self.checksum}, {self._nodes}, {self._this_node_ref}, {self._this_node_metadata})"
@@ -254,7 +228,7 @@ class FleetSensor:
         self._domain = domain
 
         self._current_state = FleetState.new(this_node)
-        self._archived_states = [self._current_state.archived()]
+        self._archived_states = deque([self._current_state.archived()], maxlen=5)
         self._remote_states = {}
         self._remote_last_seen = {}
 
@@ -267,6 +241,12 @@ class FleetSensor:
     def record_node(self, node: 'Ursula'):
 
         if node.domain == self._domain:
+            # Replace the existing object with a newer object, even if they're equal
+            # (this object can be mutated externally).
+            # This behavior is supposed to be consistent with that of the node storage
+            # (where a newer object with the same `checksum_address` replaces an older one).
+            if node in self._nodes_to_add:
+                self._nodes_to_add.remove(node)
             self._nodes_to_add.add(node)
 
             if self._auto_update_state:
@@ -339,18 +319,11 @@ class FleetSensor:
         """
         # `_archived_states` is never empty, one state is created in the constructor
         previous_states_num = min(len(self._archived_states) - 1, quantity)
-        return self._archived_states[-previous_states_num-1:-1]
+        return list(self._archived_states)[-previous_states_num-1:-1]
 
     def addresses(self):
         return self._current_state.addresses()
 
-    def snapshot(self):
-        return self._current_state.snapshot()
-
-    @staticmethod
-    def unpack_snapshot(data):
-        return FleetState.unpack_snapshot(data)
-
     def record_fleet_state(self, skip_this_node: bool = False) -> StateDiff:
         new_state, diff = self._current_state.with_updated_nodes(nodes_to_add=self._nodes_to_add,
                                                                  nodes_to_remove=self._nodes_to_remove,
@@ -380,14 +353,14 @@ class FleetSensor:
 
     def record_remote_fleet_state(self,
                                   checksum_address: ChecksumAddress,
-                                  state_checksum: str,
+                                  state_checksum: FleetStateChecksum,
                                   timestamp: maya.MayaDT,
                                   population: int):
 
         if checksum_address not in self._current_state:
             raise KeyError(f"A node {checksum_address} is not present in the current fleet state")
 
-        nickname = Nickname.from_seed(state_checksum, length=1)
+        nickname = Nickname.from_seed(bytes(state_checksum), length=1)
         state = ArchivedFleetState(checksum=state_checksum,
                                    nickname=nickname,
                                    timestamp=timestamp,
@@ -405,12 +378,12 @@ class FleetSensor:
 
         recorded_fleet_state = self._remote_states.get(node.checksum_address, None)
         last_learned_from = self._remote_last_seen.get(node.checksum_address, None)
-        worker_address = node.worker_address if node.verified_node else None
+        operator_address = node.operator_address if node.verified_node else None
 
         return RemoteUrsulaStatus(verified=node.verified_node,
                                   nickname=node.nickname,
                                   staker_address=node.checksum_address,
-                                  worker_address=worker_address,
+                                  operator_address=operator_address,
                                   rest_url=node.rest_url(),
                                   timestamp=node.timestamp,
                                   last_learned_from=last_learned_from,
@@ -422,7 +395,7 @@ class RemoteUrsulaStatus(NamedTuple):
     verified: bool
     nickname: Nickname
     staker_address: ChecksumAddress
-    worker_address: Optional[ChecksumAddress]
+    operator_address: Optional[ChecksumAddress]
     rest_url: str
     timestamp: maya.MayaDT
     recorded_fleet_state: Optional[ArchivedFleetState]
@@ -440,7 +413,7 @@ class RemoteUrsulaStatus(NamedTuple):
         return dict(verified=self.verified,
                     nickname=self.nickname.to_json(),
                     staker_address=self.staker_address,
-                    worker_address=self.worker_address,
+                    operator_address=self.operator_address,
                     rest_url=self.rest_url,
                     timestamp=self.timestamp.iso8601(),
                     recorded_fleet_state=recorded_fleet_state_json,
