@@ -19,12 +19,12 @@ from base64 import b64encode
 
 from marshmallow import fields
 
-from nucypher.characters.control.specifications.fields.base import BaseField
+from nucypher.control.specifications.fields.base import BaseField
 
 
 class Cleartext(BaseField, fields.String):
 
-    def _serialize(self, value, attr, data, **kwargs):
+    def _serialize(self, value, attr, obj, **kwargs):
         return value.decode()
 
     def _deserialize(self, value, attr, data, **kwargs):
