@@ -17,7 +17,7 @@
 
 from marshmallow import fields
 
-from nucypher.characters.control.specifications.fields.base import BaseField
+from nucypher.control.specifications.fields.base import BaseField
 
 
 class Label(BaseField, fields.Field):
