@@ -16,36 +16,25 @@
 """
 
 
+from typing import Type, Union, Dict
+
 import click
 from constant_sorrow.constants import UNKNOWN_DEVELOPMENT_CHAIN_ID
-
 from maya import MayaDT
 from tabulate import tabulate
-from typing import Type, Union, Dict
 from web3.main import Web3
 
-from nucypher.blockchain.economics import BaseEconomics
 from nucypher.blockchain.eth.deployers import BaseContractDeployer
 from nucypher.blockchain.eth.interfaces import BlockchainDeployerInterface, VersionedContract, BlockchainInterface
 from nucypher.blockchain.eth.registry import LocalContractRegistry
-from nucypher.blockchain.eth.token import NU
 from nucypher.blockchain.eth.utils import calculate_period_duration
-from nucypher.characters.control.emitters import StdoutEmitter
 from nucypher.cli.literature import (
     ABORT_DEPLOYMENT,
-    CHARACTER_DESTRUCTION,
-    CONFIRM_ENABLE_RESTAKING,
-    CONFIRM_ENABLE_WINDING_DOWN,
-    CONFIRM_LARGE_STAKE_DURATION,
-    CONFIRM_LARGE_STAKE_VALUE,
-    CONFIRM_STAGED_STAKE,
-    RESTAKING_AGREEMENT,
-    WINDING_DOWN_AGREEMENT,
-    SNAPSHOTS_DISABLING_AGREEMENT,
-    CONFIRM_DISABLE_SNAPSHOTS
+    CHARACTER_DESTRUCTION
 )
 from nucypher.cli.literature import CONFIRM_VERSIONED_UPGRADE
 from nucypher.config.base import CharacterConfiguration
+from nucypher.control.emitters import StdoutEmitter
 
 
 def confirm_deployment(emitter: StdoutEmitter, deployer_interface: BlockchainDeployerInterface) -> bool:
@@ -65,47 +54,6 @@ def confirm_deployment(emitter: StdoutEmitter, deployer_interface: BlockchainDep
     return True
 
 
-def confirm_enable_restaking(emitter: StdoutEmitter, staking_address: str) -> bool:
-    """Interactively confirm enabling of the restaking with user agreements."""
-    emitter.message(RESTAKING_AGREEMENT.format(staking_address=staking_address))
-    click.confirm(CONFIRM_ENABLE_RESTAKING.format(staking_address=staking_address), abort=True)
-    return True
-
-
-def confirm_enable_winding_down(emitter: StdoutEmitter, staking_address: str) -> bool:
-    """Interactively confirm enabling of winding down with user agreements."""
-    emitter.message(WINDING_DOWN_AGREEMENT)
-    click.confirm(CONFIRM_ENABLE_WINDING_DOWN.format(staking_address=staking_address), abort=True)
-    return True
-
-
-def confirm_disable_snapshots(emitter: StdoutEmitter, staking_address: str) -> bool:
-    """Interactively confirm disabling of taking snapshots with user agreements."""
-    emitter.message(SNAPSHOTS_DISABLING_AGREEMENT)
-    click.confirm(CONFIRM_DISABLE_SNAPSHOTS.format(staking_address=staking_address), abort=True)
-    return True
-
-
-def confirm_staged_stake(staker_address: str, value: NU, lock_periods: int) -> bool:
-    """Interactively confirm a new stake reviewing all staged stake details."""
-    click.confirm(CONFIRM_STAGED_STAKE.format(nunits=str(value.to_nunits()),
-                                              tokens=value,
-                                              staker_address=staker_address,
-                                              lock_periods=lock_periods), abort=True)
-    return True
-
-
-def confirm_large_and_or_long_stake(value: NU = None, lock_periods: int = None, economics: BaseEconomics = None) -> bool:
-    """Interactively confirm a large stake and/or a long stake duration."""
-    if economics and value and (value > (NU.from_nunits(economics.minimum_allowed_locked) * 10)):  # > 10x min stake
-        click.confirm(CONFIRM_LARGE_STAKE_VALUE.format(value=value), abort=True)
-    if economics and lock_periods and (lock_periods > economics.maximum_rewarded_periods):  # > 1 year
-        lock_days = (lock_periods * economics.hours_per_period) // 24
-        click.confirm(CONFIRM_LARGE_STAKE_DURATION.format(lock_periods=lock_periods, lock_days=lock_days),
-                      abort=True)
-    return True
-
-
 def confirm_destroy_configuration(config: CharacterConfiguration) -> bool:
     """Interactively confirm destruction of nucypher configuration files"""
     # TODO: This is a workaround for ursula - needs follow up
@@ -115,7 +63,7 @@ def confirm_destroy_configuration(config: CharacterConfiguration) -> bool:
         database = "No database found"
     confirmation = CHARACTER_DESTRUCTION.format(name=config.NAME,
                                                 root=config.config_root,
-                                                keystore=config.keyring_root,
+                                                keystore=config.keystore_dir,
                                                 nodestore=config.node_storage.source,
                                                 config=config.filepath,
                                                 database=database)
@@ -155,8 +103,8 @@ def confirm_staged_grant(emitter, grant_request: Dict, federated: bool, seconds_
         emitter.echo(tabulate(table, tablefmt="simple"))
         return
 
-    period_rate = Web3.fromWei(pretty_request['n'] * pretty_request['rate'], 'gwei')
-    pretty_request['rate'] = f"{pretty_request['rate']} wei/period * {pretty_request['n']} nodes"
+    period_rate = Web3.fromWei(pretty_request['shares'] * pretty_request['rate'], 'gwei')
+    pretty_request['rate'] = f"{pretty_request['rate']} wei/period * {pretty_request['shares']} nodes"
 
     expiration = pretty_request['expiration']
     periods = calculate_period_duration(future_time=MayaDT.from_datetime(expiration),
@@ -165,9 +113,9 @@ def confirm_staged_grant(emitter, grant_request: Dict, federated: bool, seconds_
     pretty_request['expiration'] = f"{pretty_request['expiration']} ({periods} periods)"
 
     # M of N
-    pretty_request['Threshold Shares'] = f"{pretty_request['m']} of {pretty_request['n']}"
-    del pretty_request['m']
-    del pretty_request['n']
+    pretty_request['Threshold Shares'] = f"{pretty_request['threshold']} of {pretty_request['shares']}"
+    del pretty_request['threshold']
+    del pretty_request['shares']
 
     def prettify_field(field):
         field_words = [word.capitalize() for word in field.split('_')]
