@@ -17,70 +17,37 @@
 
 import os
 from pathlib import Path
-from typing import Callable
-from typing import Optional, Tuple, Type
+from typing import Optional, Type
 
 import click
 from tabulate import tabulate
 from web3.main import Web3
 
-from nucypher.blockchain.eth.actors import StakeHolder, Staker
 from nucypher.blockchain.eth.agents import ContractAgency, NucypherTokenAgent
 from nucypher.blockchain.eth.interfaces import BlockchainInterfaceFactory
 from nucypher.blockchain.eth.networks import NetworksInventory
 from nucypher.blockchain.eth.registry import InMemoryContractRegistry, BaseContractRegistry
 from nucypher.blockchain.eth.signers.base import Signer
-from nucypher.blockchain.eth.token import NU, Stake
-from nucypher.characters.control.emitters import StdoutEmitter
+from nucypher.blockchain.eth.token import NU
 from nucypher.cli.actions.configure import get_config_filepaths
 from nucypher.cli.literature import (
     GENERIC_SELECT_ACCOUNT,
     NO_CONFIGURATIONS_ON_DISK,
     NO_ETH_ACCOUNTS,
-    NO_STAKES_FOUND,
-    ONLY_DISPLAYING_DIVISIBLE_STAKES_NOTE,
     SELECT_NETWORK,
-    SELECT_STAKE,
-    SELECT_STAKING_ACCOUNT_INDEX,
     SELECTED_ACCOUNT,
     IGNORE_OLD_CONFIGURATION,
     DEFAULT_TO_LONE_CONFIG_FILE
 )
 from nucypher.cli.painting.policies import paint_cards
-from nucypher.cli.painting.staking import paint_stakes
 from nucypher.config.base import CharacterConfiguration
-from nucypher.config.constants import NUCYPHER_ENVVAR_WORKER_ADDRESS, DEFAULT_CONFIG_ROOT
+from nucypher.config.constants import NUCYPHER_ENVVAR_OPERATOR_ADDRESS, DEFAULT_CONFIG_ROOT
+from nucypher.control.emitters import StdoutEmitter
 from nucypher.policy.identity import Card
 
 
-def select_stake(staker: Staker,
-                 emitter: StdoutEmitter,
-                 stakes_status: Stake.Status = Stake.Status.EDITABLE,
-                 filter_function: Callable[[Stake], bool] = None
-                 ) -> Stake:
-    """Interactively select a stake or abort if there are no eligible stakes."""
-
-    if stakes_status.is_child(Stake.Status.DIVISIBLE):
-        emitter.echo(ONLY_DISPLAYING_DIVISIBLE_STAKES_NOTE, color='yellow')
-
-    # Filter stakes by status
-    stakes = staker.sorted_stakes(parent_status=stakes_status, filter_function=filter_function)
-    if not stakes:
-        emitter.echo(NO_STAKES_FOUND, color='red')
-        raise click.Abort
-
-    # Interactive Selection
-    paint_unlocked = stakes_status.is_child(Stake.Status.UNLOCKED)
-    paint_stakes(staker=staker, emitter=emitter, stakes=stakes, paint_unlocked=paint_unlocked)
-    indexed_stakes = {stake.index: stake for stake in stakes}
-    indices = [str(index) for index in indexed_stakes.keys()]
-    choice = click.prompt(SELECT_STAKE, type=click.Choice(indices))
-    chosen_stake = indexed_stakes[int(choice)]
-    return chosen_stake
-
-
 def select_client_account(emitter,
-                          provider_uri: str = None,
+                          eth_provider_uri: str = None,
                           signer: Signer = None,
                           signer_uri: str = None,
                           prompt: str = None,
@@ -101,17 +68,17 @@ def select_client_account(emitter,
     if signer and signer_uri:
         raise ValueError('Pass either signer or signer_uri but not both.')
 
-    if not any((provider_uri, signer_uri, signer)):
+    if not any((eth_provider_uri, signer_uri, signer)):
         raise ValueError("At least a provider URI, signer URI or signer must be provided to select an account")
 
-    if provider_uri:
+    if eth_provider_uri:
         # Connect to the blockchain in order to select an account
-        if not BlockchainInterfaceFactory.is_interface_initialized(provider_uri=provider_uri):
-            BlockchainInterfaceFactory.initialize_interface(provider_uri=provider_uri, poa=poa, emitter=emitter)
+        if not BlockchainInterfaceFactory.is_interface_initialized(eth_provider_uri=eth_provider_uri):
+            BlockchainInterfaceFactory.initialize_interface(eth_provider_uri=eth_provider_uri, poa=poa, emitter=emitter)
         if not signer_uri:
-            signer_uri = provider_uri
+            signer_uri = eth_provider_uri
 
-    blockchain = BlockchainInterfaceFactory.get_interface(provider_uri=provider_uri)
+    blockchain = BlockchainInterfaceFactory.get_interface(eth_provider_uri=eth_provider_uri)
 
     if signer_uri and not signer:
         testnet = network != NetworksInventory.MAINNET
@@ -154,7 +121,7 @@ def select_client_account(emitter,
             row.append(f'{ether_balance} ETH')
         if show_nu_balance:
             token_agent = ContractAgency.get_agent(NucypherTokenAgent, registry=registry)
-            token_balance = NU.from_nunits(token_agent.get_balance(account, registry))
+            token_balance = NU.from_units(token_agent.get_balance(account, registry))
             row.append(token_balance)
         rows.append(row)
     emitter.echo(tabulate(rows, headers=headers, showindex='always'))
@@ -169,38 +136,11 @@ def select_client_account(emitter,
     return chosen_account
 
 
-def select_client_account_for_staking(emitter: StdoutEmitter,
-                                      stakeholder: StakeHolder,
-                                      staking_address: Optional[str],
-                                      ) -> Tuple[str, str]:
-    """
-    Manages client account selection for stake-related operations.
-    It always returns a tuple of addresses: the first is the local client account and the second is the staking address.
-
-    When this is not a preallocation staker (which is the normal use case), both addresses are the same.
-    Otherwise, when the staker is a contract managed by a beneficiary account,
-    then the local client account is the beneficiary, and the staking address is the address of the staking contract.
-    """
-
-    if staking_address:
-        client_account = staking_address
-    else:
-        client_account = select_client_account(prompt=SELECT_STAKING_ACCOUNT_INDEX,
-                                               emitter=emitter,
-                                               registry=stakeholder.registry,
-                                               network=stakeholder.domain,
-                                               signer=stakeholder.signer)
-        staking_address = client_account
-    stakeholder.assimilate(client_account)
-
-    return client_account, staking_address
-
-
-def select_network(emitter: StdoutEmitter) -> str:
+def select_network(emitter: StdoutEmitter, message: Optional[str] = None) -> str:
     """Interactively select a network from nucypher networks inventory list"""
-    headers = ["Network"]
+    emitter.message(message=message or str(), color='yellow')
     rows = [[n] for n in NetworksInventory.NETWORKS]
-    emitter.echo(tabulate(rows, headers=headers, showindex='always'))
+    emitter.echo(tabulate(rows, showindex='always'))
     choice = click.prompt(SELECT_NETWORK, default=0, type=click.IntRange(0, len(NetworksInventory.NETWORKS)-1))
     network = NetworksInventory.NETWORKS[choice]
     return network
@@ -208,16 +148,16 @@ def select_network(emitter: StdoutEmitter) -> str:
 
 def select_config_file(emitter: StdoutEmitter,
                        config_class: Type[CharacterConfiguration],
-                       config_root: str = None,
+                       config_root: Optional[Path] = None,
                        checksum_address: str = None,
-                       ) -> str:
+                       ) -> Path:
     """
     Selects a nucypher character configuration file from the disk automatically or interactively.
 
     Behaviour
     ~~~~~~~~~
 
-    - If checksum address is supplied by parameter or worker address env var - confirm there is a corresponding
+    - If checksum address is supplied by parameter or operator address env var - confirm there is a corresponding
       file on the disk or raise ValueError.
 
     - If there is only one configuration file for the character, automatically return its filepath.
@@ -236,7 +176,7 @@ def select_config_file(emitter: StdoutEmitter,
                                                          command=config_class.NAME), color='red')
         raise click.Abort()
 
-    checksum_address = checksum_address or os.environ.get(NUCYPHER_ENVVAR_WORKER_ADDRESS, None)  # TODO: Deprecate worker_address in favor of checksum_address
+    checksum_address = checksum_address or os.environ.get(NUCYPHER_ENVVAR_OPERATOR_ADDRESS, None)  # TODO: Deprecate operator_address in favor of checksum_address
 
     parsed_config_files = list()
     parsed_addresses_and_filenames = list()
@@ -295,7 +235,7 @@ def select_config_file(emitter: StdoutEmitter,
 def select_card(emitter, card_identifier: str) -> Card:
     if not card_identifier:
         cards = []
-        for filename in os.listdir(Card.CARD_DIR):
+        for filename in Card.CARD_DIR.iterdir():
             filepath = Card.CARD_DIR / filename
             card = Card.load(filepath=filepath)
             cards.append(card)
