@@ -14,11 +14,11 @@ GNU Affero General Public License for more details.
 You should have received a copy of the GNU Affero General Public License
 along with nucypher.  If not, see <https://www.gnu.org/licenses/>.
 """
-
+from pathlib import Path
 
 import click
 
-from nucypher.characters.control.emitters import StdoutEmitter
+from nucypher.control.emitters import StdoutEmitter
 from nucypher.characters.control.interfaces import AliceInterface
 from nucypher.cli.actions.auth import get_nucypher_password
 from nucypher.cli.actions.collect import collect_bob_public_keys, collect_policy_parameters
@@ -44,33 +44,32 @@ from nucypher.cli.options import (
     option_force,
     option_hw_wallet,
     option_light,
-    option_m,
     option_middleware,
     option_min_stake,
-    option_n,
     option_network,
+    option_shares,
     option_poa,
-    option_provider_uri,
+    option_eth_provider_uri,
     option_registry_filepath,
     option_signer_uri,
     option_teacher_uri,
+    option_threshold,
     option_lonely,
-    option_max_gas_price
+    option_max_gas_price,
+    option_key_material, option_payment_method, option_payment_network, option_payment_provider
 )
 from nucypher.cli.painting.help import paint_new_installation_help
 from nucypher.cli.painting.policies import paint_single_card
 from nucypher.cli.types import EIP55_CHECKSUM_ADDRESS
 from nucypher.cli.utils import make_cli_character, setup_emitter
 from nucypher.config.characters import AliceConfiguration
-from nucypher.config.constants import (
-    TEMPORARY_DOMAIN,
-)
-from nucypher.config.keyring import NucypherKeyring
+from nucypher.config.constants import TEMPORARY_DOMAIN
+from nucypher.crypto.keystore import Keystore
 from nucypher.network.middleware import RestMiddleware
 from nucypher.policy.identity import Card
 
 option_pay_with = click.option('--pay-with', help="Run with a specified account", type=EIP55_CHECKSUM_ADDRESS)
-option_payment_periods = click.option('--payment-periods', help="Policy payment periods", type=click.INT)
+option_duration = click.option('--duration', help="Policy payment periods", type=click.INT)
 
 
 class AliceConfigOptions:
@@ -80,21 +79,24 @@ class AliceConfigOptions:
     def __init__(self,
                  dev: bool,
                  network: str,
-                 provider_uri: str,
+                 eth_provider_uri: str,
                  federated_only: bool,
                  discovery_port: int,
                  pay_with: str,
-                 registry_filepath: str,
+                 registry_filepath: Path,
                  middleware: RestMiddleware,
                  gas_strategy: str,
                  max_gas_price: int,  # gwei
                  signer_uri: str,
                  lonely: bool,
+                 payment_method: str,
+                 payment_provider: str,
+                 payment_network: str
                  ):
 
         self.dev = dev
         self.domain = network
-        self.provider_uri = provider_uri
+        self.eth_provider_uri = eth_provider_uri
         self.signer_uri = signer_uri
         self.gas_strategy = gas_strategy
         self.max_gas_price = max_gas_price
@@ -104,6 +106,10 @@ class AliceConfigOptions:
         self.registry_filepath = registry_filepath
         self.middleware = middleware
         self.lonely = lonely
+        
+        self.payment_method = payment_method
+        self.payment_provider = payment_provider
+        self.payment_network = payment_network
 
     def create_config(self, emitter, config_file):
 
@@ -120,12 +126,15 @@ class AliceConfigOptions:
                 dev_mode=True,
                 network_middleware=self.middleware,
                 domain=TEMPORARY_DOMAIN,
-                provider_uri=self.provider_uri,
+                eth_provider_uri=self.eth_provider_uri,
                 signer_uri=self.signer_uri,
                 gas_strategy=self.gas_strategy,
                 max_gas_price=self.max_gas_price,
                 federated_only=True,
-                lonely=self.lonely
+                lonely=self.lonely,
+                payment_method=self.payment_method,
+                payment_provider=self.payment_provider,
+                payment_network=self.payment_network
             )
 
         else:
@@ -139,7 +148,7 @@ class AliceConfigOptions:
                     dev_mode=False,
                     network_middleware=self.middleware,
                     domain=self.domain,
-                    provider_uri=self.provider_uri,
+                    eth_provider_uri=self.eth_provider_uri,
                     signer_uri=self.signer_uri,
                     gas_strategy=self.gas_strategy,
                     max_gas_price=self.max_gas_price,
@@ -147,7 +156,10 @@ class AliceConfigOptions:
                     rest_port=self.discovery_port,
                     checksum_address=self.pay_with,
                     registry_filepath=self.registry_filepath,
-                    lonely=self.lonely
+                    lonely=self.lonely,
+                    payment_method=self.payment_method,
+                    payment_provider=self.payment_provider,
+                    payment_network=self.payment_network
                 )
             except FileNotFoundError:
                 return handle_missing_configuration_file(
@@ -160,7 +172,7 @@ group_config_options = group_options(
     AliceConfigOptions,
     dev=option_dev,
     network=option_network(),
-    provider_uri=option_provider_uri(),
+    eth_provider_uri=option_eth_provider_uri(),
     signer_uri=option_signer_uri,
     gas_strategy=option_gas_strategy,
     max_gas_price=option_max_gas_price,
@@ -170,6 +182,9 @@ group_config_options = group_options(
     registry_filepath=option_registry_filepath,
     middleware=option_middleware,
     lonely=option_lonely,
+    payment_provider=option_payment_provider,
+    payment_network=option_payment_network,
+    payment_method=option_payment_method,
 )
 
 
@@ -177,62 +192,71 @@ class AliceFullConfigOptions:
 
     __option_name__ = 'full_config_options'
 
-    def __init__(self, config_options, poa: bool, light: bool, m: int, n: int, payment_periods: int):
+    def __init__(self, config_options, poa: bool, light: bool, threshold: int, shares: int, duration: int):
         self.config_options = config_options
         self.poa = poa
         self.light = light
-        self.m = m
-        self.n = n
-        self.payment_periods = payment_periods
+        self.threshold = threshold
+        self.shares = shares
+        self.duration = duration
 
-    def generate_config(self, emitter: StdoutEmitter, config_root: str) -> AliceConfiguration:
+    def generate_config(self, emitter: StdoutEmitter, config_root: Path, key_material: str) -> AliceConfiguration:
 
         opts = self.config_options
 
         if opts.dev:
             raise click.BadArgumentUsage("Cannot create a persistent development character")
 
-        if not opts.provider_uri and not opts.federated_only:
+        if not opts.eth_provider_uri and not opts.federated_only:
             raise click.BadOptionUsage(
-                option_name='--provider',
-                message="--provider is required to create a new decentralized alice.")
+                option_name='--eth-provider',
+                message="--eth-provider is required to create a new decentralized alice.")
 
         pay_with = opts.pay_with
         if not pay_with and not opts.federated_only:
             pay_with = select_client_account(emitter=emitter,
-                                             provider_uri=opts.provider_uri,
+                                             eth_provider_uri=opts.eth_provider_uri,
                                              signer_uri=opts.signer_uri,
                                              show_eth_balance=True,
                                              network=opts.domain)
 
         return AliceConfiguration.generate(
-            password=get_nucypher_password(confirm=True),
+            password=get_nucypher_password(emitter=emitter, confirm=True),
+            key_material=bytes.fromhex(key_material) if key_material else None,
             config_root=config_root,
             checksum_address=pay_with,
             domain=opts.domain,
             federated_only=opts.federated_only,
-            provider_uri=opts.provider_uri,
+            eth_provider_uri=opts.eth_provider_uri,
             signer_uri=opts.signer_uri,
             registry_filepath=opts.registry_filepath,
             poa=self.poa,
             light=self.light,
-            m=self.m,
-            n=self.n,
-            payment_periods=self.payment_periods)
+            threshold=self.threshold,
+            shares=self.shares,
+            duration=self.duration,
+            payment_provider=opts.payment_provider,
+            payment_network=opts.payment_network,
+            payment_method=opts.payment_method,
+        )
 
     def get_updates(self) -> dict:
         opts = self.config_options
         payload = dict(checksum_address=opts.pay_with,
                        domain=opts.domain,
                        federated_only=opts.federated_only,
-                       provider_uri=opts.provider_uri,
+                       eth_provider_uri=opts.eth_provider_uri,
                        signer_uri=opts.signer_uri,
                        registry_filepath=opts.registry_filepath,
                        poa=self.poa,
                        light=self.light,
-                       m=self.m,
-                       n=self.n,
-                       payment_periods=self.payment_periods)
+                       threshold=self.threshold,
+                       shares=self.shares,
+                       duration=self.duration,
+                       payment_provider=opts.payment_provider,
+                       payment_network=opts.payment_network,
+                       payment_method=opts.payment_method,
+                       )
         # Depends on defaults being set on Configuration classes, filtrates None values
         updates = {k: v for k, v in payload.items() if v is not None}
         return updates
@@ -243,9 +267,9 @@ group_full_config_options = group_options(
     config_options=group_config_options,
     poa=option_poa,
     light=option_light,
-    m=option_m,
-    n=option_n,
-    payment_periods=option_payment_periods
+    threshold=option_threshold,
+    shares=option_shares,
+    duration=option_duration
 )
 
 
@@ -264,7 +288,7 @@ class AliceCharacterOptions:
         try:
             ALICE = make_cli_character(character_config=config,
                                        emitter=emitter,
-                                       unlock_keyring=not config.dev_mode,
+                                       unlock_keystore=not config.dev_mode,
                                        unlock_signer=not config.federated_only,
                                        teacher_uri=self.teacher_uri,
                                        min_stake=self.min_stake,
@@ -272,7 +296,7 @@ class AliceCharacterOptions:
                                        lonely=self.config_options.lonely,
                                        json_ipc=json_ipc)
             return ALICE
-        except NucypherKeyring.AuthenticationFailed as e:
+        except Keystore.AuthenticationFailed as e:
             emitter.echo(str(e), color='red', bold=True)
             click.get_current_context().exit(1)
 
@@ -295,12 +319,15 @@ def alice():
 @group_full_config_options
 @option_config_root
 @group_general_config
-def init(general_config, full_config_options, config_root):
+@option_key_material
+def init(general_config, full_config_options, config_root, key_material):
     """Create a brand new persistent Alice."""
     emitter = setup_emitter(general_config)
     if not config_root:
         config_root = general_config.config_root
-    new_alice_config = full_config_options.generate_config(emitter, config_root)
+    new_alice_config = full_config_options.generate_config(emitter=emitter,
+                                                           config_root=config_root,
+                                                           key_material=key_material)
     filepath = new_alice_config.to_configuration_file()
     paint_new_installation_help(emitter, new_configuration=new_alice_config, filepath=filepath)
 
@@ -362,7 +389,7 @@ def run(general_config, character_options, config_file, controller_port, dry_run
             controller = ALICE.make_web_controller(crash_on_error=general_config.debug)
             ALICE.log.info('Starting HTTP Character Web Controller')
             emitter.message(f'Running HTTP Alice Controller at http://localhost:{controller_port}')
-            return controller.start(http_port=controller_port, dry_run=dry_run)
+            return controller.start(port=controller_port, dry_run=dry_run)
 
     # Handle Crash
     except Exception as e:
@@ -434,7 +461,8 @@ def grant(general_config,
           value,
           rate,
           expiration,
-          m, n,
+          threshold,
+          shares,
           character_options,
           config_file,
           force):
@@ -475,8 +503,8 @@ def grant(general_config,
             force=force,
             bob_identifier=bob_public_keys.verifying_key[:8],
             label=label,
-            m=m,
-            n=n,
+            threshold=threshold,
+            shares=shares,
             rate=rate,
             value=value,
             expiration=expiration
@@ -486,8 +514,8 @@ def grant(general_config,
         'bob_encrypting_key': bob_public_keys.encrypting_key,
         'bob_verifying_key': bob_public_keys.verifying_key,
         'label': policy.label,
-        'm': policy.m,
-        'n': policy.n,
+        'threshold': policy.threshold,
+        'shares': policy.shares,
         'expiration': policy.expiration,
     }
     if not ALICE.federated_only:
