@@ -17,13 +17,14 @@ along with nucypher.  If not, see <https://www.gnu.org/licenses/>.
 
 
 import json
+from pathlib import Path
 
 import click
 from constant_sorrow import constants
 from typing import Tuple
 
-from nucypher.blockchain.eth.actors import ContractAdministrator, Trustee
-from nucypher.blockchain.eth.agents import ContractAgency, MultiSigAgent
+from nucypher.blockchain.eth.actors import ContractAdministrator
+from nucypher.blockchain.eth.agents import ContractAgency
 from nucypher.blockchain.eth.clients import PUBLIC_CHAINS
 from nucypher.blockchain.eth.constants import STAKING_ESCROW_CONTRACT_NAME
 from nucypher.blockchain.eth.interfaces import BlockchainInterface
@@ -37,7 +38,7 @@ from nucypher.blockchain.eth.registry import (
 from nucypher.blockchain.eth.signers.base import Signer
 from nucypher.blockchain.eth.signers.software import ClefSigner
 from nucypher.blockchain.eth.sol.__conf__ import SOLIDITY_COMPILER_VERSION
-from nucypher.characters.control.emitters import StdoutEmitter
+from nucypher.control.emitters import StdoutEmitter
 from nucypher.cli.actions.auth import get_client_password
 from nucypher.cli.actions.confirm import confirm_deployment, verify_upgrade_details
 from nucypher.cli.actions.select import select_client_account
@@ -62,7 +63,6 @@ from nucypher.cli.literature import (
     SUCCESSFUL_REGISTRY_DOWNLOAD,
     SUCCESSFUL_RETARGET,
     SUCCESSFUL_RETARGET_TX_BUILT,
-    SUCCESSFUL_SAVE_MULTISIG_TX_PROPOSAL,
     SUCCESSFUL_UPGRADE,
     UNKNOWN_CONTRACT_NAME,
     DEPLOYER_IS_NOT_OWNER,
@@ -78,7 +78,7 @@ from nucypher.cli.options import (
     option_hw_wallet,
     option_network,
     option_poa,
-    option_provider_uri,
+    option_eth_provider_uri,
     option_signer_uri,
     option_parameters, option_gas_strategy, option_max_gas_price)
 from nucypher.cli.painting.deployment import (
@@ -87,7 +87,6 @@ from nucypher.cli.painting.deployment import (
     paint_staged_deployment
 )
 from nucypher.cli.painting.help import echo_solidity_version
-from nucypher.cli.painting.multisig import paint_multisig_proposed_transaction
 from nucypher.cli.painting.transactions import paint_receipt_summary
 from nucypher.cli.types import EIP55_CHECKSUM_ADDRESS, EXISTING_READABLE_FILE, WEI
 from nucypher.cli.utils import (
@@ -100,7 +99,8 @@ from nucypher.crypto.powers import TransactingPower
 
 option_deployer_address = click.option('--deployer-address', help="Deployer's checksum address", type=EIP55_CHECKSUM_ADDRESS)
 option_registry_infile = click.option('--registry-infile', help="Input path for contract registry file", type=EXISTING_READABLE_FILE)
-option_registry_outfile = click.option('--registry-outfile', help="Output path for contract registry file", type=click.Path(file_okay=True))
+option_registry_outfile = click.option('--registry-outfile', help="Output path for contract registry file",
+                                       type=click.Path(dir_okay=False, path_type=Path))
 option_target_address = click.option('--target-address', help="Address of the target contract", type=EIP55_CHECKSUM_ADDRESS)
 option_gas = click.option('--gas', help="Operate with a specified gas per-transaction limit", type=click.IntRange(min=1))
 option_ignore_deployed = click.option('--ignore-deployed', help="Ignore already deployed contracts if exist.", is_flag=True)
@@ -113,16 +113,16 @@ class ActorOptions:
     __option_name__ = 'actor_options'
 
     def __init__(self,
-                 provider_uri: str,
+                 eth_provider_uri: str,
                  deployer_address: str,
                  contract_name: str,
-                 registry_infile: str,
-                 registry_outfile: str,
+                 registry_infile: Path,
+                 registry_outfile: Path,
                  hw_wallet: bool,
                  dev: bool,
                  force: bool,
                  poa: bool,
-                 config_root: str,
+                 config_root: Path,
                  etherscan: bool,
                  ignore_solidity_check,
                  gas_strategy: str,
@@ -131,7 +131,7 @@ class ActorOptions:
                  network: str
                  ):
 
-        self.provider_uri = provider_uri
+        self.eth_provider_uri = eth_provider_uri
         self.signer_uri = signer_uri
         self.gas_strategy = gas_strategy
         self.max_gas_price = max_gas_price
@@ -150,12 +150,11 @@ class ActorOptions:
 
     def create_actor(self,
                      emitter: StdoutEmitter,
-                     is_multisig: bool = False
                      ) -> Tuple[ContractAdministrator, str, BlockchainInterface, BaseContractRegistry]:
 
         ensure_config_root(self.config_root)
         deployer_interface = initialize_deployer_interface(poa=self.poa,
-                                                           provider_uri=self.provider_uri,
+                                                           eth_provider_uri=self.eth_provider_uri,
                                                            emitter=emitter,
                                                            ignore_solidity_check=self.ignore_solidity_check,
                                                            gas_strategy=self.gas_strategy,
@@ -179,36 +178,30 @@ class ActorOptions:
         #
 
         # Verify Address & collect password
-        if is_multisig:
-            multisig_agent = ContractAgency.get_agent(MultiSigAgent, registry=local_registry)
-            deployer_address = multisig_agent.contract.address
-            transacting_power = None
+        testnet = deployer_interface.client.chain_name != PUBLIC_CHAINS[1]  # Mainnet
+        signer = Signer.from_signer_uri(self.signer_uri, testnet=testnet)
+        deployer_address = self.deployer_address
+        if not deployer_address:
+            deployer_address = select_client_account(emitter=emitter,
+                                                     prompt=SELECT_DEPLOYER_ACCOUNT,
+                                                     registry=local_registry,
+                                                     eth_provider_uri=self.eth_provider_uri,
+                                                     signer=signer,
+                                                     show_eth_balance=True)
 
-        else:
-            testnet = deployer_interface.client.chain_name != PUBLIC_CHAINS[1]  # Mainnet
-            signer = Signer.from_signer_uri(self.signer_uri, testnet=testnet)
-            deployer_address = self.deployer_address
-            if not deployer_address:
-                deployer_address = select_client_account(emitter=emitter,
-                                                         prompt=SELECT_DEPLOYER_ACCOUNT,
-                                                         registry=local_registry,
-                                                         provider_uri=self.provider_uri,
-                                                         signer=signer,
-                                                         show_eth_balance=True)
+        if not self.force:
+            click.confirm(CONFIRM_SELECTED_ACCOUNT.format(address=deployer_address), abort=True)
 
-            if not self.force:
-                click.confirm(CONFIRM_SELECTED_ACCOUNT.format(address=deployer_address), abort=True)
-
-            # Authenticate
-            is_clef = ClefSigner.is_valid_clef_uri(self.signer_uri)
-            password_required = all((not is_clef,
-                                     not signer.is_device(account=deployer_address),
-                                     not deployer_interface.client.is_local,
-                                     not self.hw_wallet))
-            if password_required:
-                password = get_client_password(checksum_address=deployer_address)
-                signer.unlock_account(password=password, account=deployer_address)
-            transacting_power = TransactingPower(signer=signer, account=deployer_address)
+        # Authenticate
+        is_clef = ClefSigner.is_valid_clef_uri(self.signer_uri)
+        password_required = all((not is_clef,
+                                 not signer.is_device(account=deployer_address),
+                                 not deployer_interface.client.is_local,
+                                 not self.hw_wallet))
+        if password_required:
+            password = get_client_password(checksum_address=deployer_address)
+            signer.unlock_account(password=password, account=deployer_address)
+        transacting_power = TransactingPower(signer=signer, account=deployer_address)
 
         # Produce Actor
         ADMINISTRATOR = ContractAdministrator(registry=local_registry,
@@ -225,7 +218,7 @@ class ActorOptions:
 
 group_actor_options = group_options(
     ActorOptions,
-    provider_uri=option_provider_uri(),
+    eth_provider_uri=option_eth_provider_uri(),
     gas_strategy=option_gas_strategy,
     max_gas_price=option_max_gas_price,
     signer_uri=option_signer_uri,
@@ -289,20 +282,20 @@ def download_registry(general_config, config_root, registry_outfile, network, fo
 
 @deploy.command()
 @group_general_config
-@option_provider_uri(required=True)
+@option_eth_provider_uri(required=True)
 @option_config_root
 @option_registry_infile
 @option_deployer_address
 @option_poa
 @option_network(required=False, default=NetworksInventory.DEFAULT)
 @option_ignore_solidity_version
-def inspect(general_config, provider_uri, config_root, registry_infile, deployer_address,
+def inspect(general_config, eth_provider_uri, config_root, registry_infile, deployer_address,
             poa, ignore_solidity_check, network):
     """Echo owner information and bare contract metadata."""
     emitter = general_config.emitter
     ensure_config_root(config_root)
     initialize_deployer_interface(poa=poa,
-                                  provider_uri=provider_uri,
+                                  eth_provider_uri=eth_provider_uri,
                                   emitter=emitter,
                                   ignore_solidity_check=ignore_solidity_check)
     download_required = not bool(registry_infile)
@@ -322,8 +315,7 @@ def inspect(general_config, provider_uri, config_root, registry_infile, deployer
 @option_ignore_deployed
 @option_confirmations
 @click.option('--retarget', '-d', help="Retarget a contract's proxy.", is_flag=True)
-@click.option('--multisig', help="Build raw transaction for upgrade via MultiSig ", is_flag=True)
-def upgrade(general_config, actor_options, retarget, target_address, ignore_deployed, multisig, confirmations):
+def upgrade(general_config, actor_options, retarget, target_address, ignore_deployed, confirmations):
     """Upgrade NuCypher existing proxy contract deployments."""
 
     #
@@ -331,7 +323,7 @@ def upgrade(general_config, actor_options, retarget, target_address, ignore_depl
     #
 
     emitter = general_config.emitter
-    ADMINISTRATOR, deployer_address, blockchain, local_registry = actor_options.create_actor(emitter, is_multisig=bool(multisig))  # FIXME: Workaround for building MultiSig TXs | NRN
+    ADMINISTRATOR, deployer_address, blockchain, local_registry = actor_options.create_actor(emitter)
 
     #
     # Pre-flight
@@ -363,40 +355,7 @@ def upgrade(general_config, actor_options, retarget, target_address, ignore_depl
     # Business
     #
 
-    if multisig:
-        if not target_address:
-            raise click.BadArgumentUsage(message="--multisig requires using --target-address.")
-        if not actor_options.force:
-            click.confirm(CONFIRM_BUILD_RETARGET_TRANSACTION.format(contract_name=contract_name,
-                                                                    target_address=target_address), abort=True)
-        transaction = ADMINISTRATOR.retarget_proxy(contract_name=contract_name,
-                                                   target_address=target_address,
-                                                   just_build_transaction=True,
-                                                   confirmations=confirmations)
-
-        trustee_address = select_client_account(emitter=emitter,
-                                                prompt="Select trustee address",
-                                                provider_uri=actor_options.provider_uri,
-                                                show_eth_balance=False,
-                                                show_nu_balance=False,
-                                                show_staking=False)
-
-        if not actor_options.force:
-            click.confirm(CONFIRM_SELECTED_ACCOUNT.format(address=trustee_address), abort=True)
-
-        trustee = Trustee(registry=local_registry, checksum_address=trustee_address)
-        transaction_proposal = trustee.create_transaction_proposal(transaction)
-
-        message = SUCCESSFUL_RETARGET_TX_BUILT.format(contract_name=contract_name, target_address=target_address)
-        emitter.message(message, color='green')
-        paint_multisig_proposed_transaction(emitter, transaction_proposal)  # TODO: Show decoded function too
-
-        filepath = f'proposal-{trustee.multisig_agent.contract_address[:8]}-TX-{transaction_proposal.nonce}.json'
-        transaction_proposal.write(filepath=filepath)
-        emitter.echo(SUCCESSFUL_SAVE_MULTISIG_TX_PROPOSAL.format(filepath=filepath), color='blue', bold=True)
-        return  # Exit
-
-    elif retarget:
+    if retarget:
         if not target_address:
             raise click.BadArgumentUsage(message="--target-address is required when using --retarget")
         if not actor_options.force:
@@ -575,28 +534,3 @@ def transfer_ownership(general_config, actor_options, target_address, gas):
                                                    new_owner=target_address,
                                                    transaction_gas_limit=gas)
     paint_receipt_summary(emitter=emitter, receipt=receipt)
-
-
-@deploy.command("set-range")
-@group_general_config
-@group_actor_options
-@click.option('--minimum', help="Minimum value for range (in wei)", type=WEI)
-@click.option('--default', help="Default value for range (in wei)", type=WEI)
-@click.option('--maximum', help="Maximum value for range (in wei)", type=WEI)
-def set_range(general_config, actor_options, minimum, default, maximum):
-    """
-    Set the minimum, default & maximum fee rate for all policies ('global fee range') in the policy manager contract.
-    The minimum acceptable fee rate (set by stakers) must fall within the global fee range.
-    """
-    emitter = general_config.emitter
-    ADMINISTRATOR, _, _, _ = actor_options.create_actor(emitter)
-
-    if not minimum:
-        minimum = click.prompt("Enter new minimum value for range", type=click.IntRange(min=0))
-    if not default:
-        default = click.prompt("Enter new default value for range", type=click.IntRange(min=minimum))
-    if not maximum:
-        maximum = click.prompt("Enter new maximum value for range", type=click.IntRange(min=default))
-
-    ADMINISTRATOR.set_fee_rate_range(minimum=minimum, default=default, maximum=maximum)
-    emitter.echo(MINIMUM_POLICY_RATE_EXCEEDED_WARNING.format(minimum=minimum, maximum=maximum, default=default))
