@@ -17,7 +17,8 @@ along with nucypher.  If not, see <https://www.gnu.org/licenses/>.
 
 
 import click
-from umbral.keys import UmbralPublicKey
+
+from nucypher_core.umbral import PublicKey
 
 from nucypher.characters.control.interfaces import EnricoInterface
 from nucypher.characters.lawful import Enrico
@@ -53,7 +54,7 @@ def run(general_config, policy_encrypting_key, dry_run, http_port):
 
     ENRICO.log.info('Starting HTTP Character Web Controller')
     controller = ENRICO.make_web_controller()
-    return controller.start(http_port=http_port, dry_run=dry_run)
+    return controller.start(port=http_port, dry_run=dry_run)
 
 
 @enrico.command()
@@ -99,7 +100,7 @@ def encrypt(general_config, policy_encrypting_key, message, file, ipfs):
 
 
 def _create_enrico(emitter, policy_encrypting_key) -> Enrico:
-    policy_encrypting_key = UmbralPublicKey.from_bytes(bytes.fromhex(policy_encrypting_key))
+    policy_encrypting_key = PublicKey.from_bytes(bytes.fromhex(policy_encrypting_key))
     ENRICO = Enrico(policy_encrypting_key=policy_encrypting_key)
     ENRICO.controller.emitter = emitter
     return ENRICO
