@@ -20,16 +20,12 @@ import click
 from nucypher.cli.commands import (
     alice,
     bob,
-    dao,
     enrico,
-    felix,
-    multisig,
-    stake,
     status,
     ursula,
-    worklock,
-    cloudworkers,
-    contacts
+    contacts,
+    porter,
+    bond,
 )
 from nucypher.cli.painting.help import echo_version, echo_config_root_path, echo_logging_root_path
 
@@ -51,9 +47,9 @@ def nucypher_cli():
 #
 #             ursula
 #               |
-#               |
-#               |
-#               |
+#               |  bond
+#               |  /
+#               | /
 # stdin --> cli.main --- alice
 #               | \
 #               |  \
@@ -72,20 +68,22 @@ def nucypher_cli():
 ENTRY_POINTS = (
 
     # Characters & Actors
-    alice.alice,        # Author of Policies
-    bob.bob,            # Builder of Capsules
-    enrico.enrico,      # Encryptor of Data
-    ursula.ursula,      # Untrusted Re-Encryption Proxy
-    stake.stake,        # Stake Management
-    worklock.worklock,  # WorkLock
+    ursula.ursula,  # Untrusted Re-Encryption Proxy
+    enrico.enrico,  # Encryptor of Data
+
+    # PRE Application
+    bond.bond,
+    bond.unbond,
 
     # Utility Commands
-    status.status,      # Network Status
-    dao.dao,            # NuCypher DAO
-    multisig.multisig,  # MultiSig operations
-    felix.felix,        # Faucet
-    cloudworkers.cloudworkers,  # Remote Worker node management
-    contacts.contacts,          # Character "card" management
+    status.status,      # Network status explorer
+    porter.porter,      # Network support services
+
+    # Demos
+    alice.alice,        # Author of Policies
+    bob.bob,            # Builder of Capsules
+    contacts.contacts,  # "character card" management
+
 )
 
 for entry_point in ENTRY_POINTS:
