@@ -16,10 +16,11 @@ along with nucypher.  If not, see <https://www.gnu.org/licenses/>.
 """
 
 
+import functools
 from collections import namedtuple
+from pathlib import Path
 
 import click
-import functools
 
 from nucypher.blockchain.eth.constants import NUCYPHER_CONTRACT_NAMES
 from nucypher.cli.types import (
@@ -30,7 +31,7 @@ from nucypher.cli.types import (
     NuCypherNetworkName,
     WEI,
     STAKED_TOKENS_RANGE,
-    MIN_ALLOWED_LOCKED_TOKENS
+    MIN_AUTHORIZATION, PAYMENT_METHOD_CHOICES
 )
 from nucypher.utilities.logging import Logger
 
@@ -38,9 +39,9 @@ from nucypher.utilities.logging import Logger
 
 option_checksum_address = click.option('--checksum-address', help="Run with a specified account", type=EIP55_CHECKSUM_ADDRESS)
 option_config_file = click.option('--config-file', help="Path to configuration file", type=EXISTING_READABLE_FILE)
-option_config_root = click.option('--config-root', help="Custom configuration directory", type=click.Path())
+option_config_root = click.option('--config-root', help="Custom configuration directory", type=click.Path(path_type=Path))
 option_dev = click.option('--dev', '-d', help="Enable development mode", is_flag=True)
-option_db_filepath = click.option('--db-filepath', help="The database filepath to connect to", type=click.STRING)
+option_db_filepath = click.option('--db-filepath', help="The database filepath to connect to", type=click.Path(path_type=Path))
 option_dry_run = click.option('--dry-run', '-x', help="Execute normally without actually starting the node", is_flag=True)
 option_etherscan = click.option('--etherscan/--no-etherscan', help="Enable/disable viewing TX in Etherscan")
 option_event_name = click.option('--event-name', help="Specify an event by name", type=click.STRING)
@@ -48,20 +49,27 @@ option_federated_only = click.option('--federated-only/--decentralized', '-F', h
 option_force = click.option('--force', help="Don't ask for confirmation", is_flag=True)
 option_gas_price = click.option('--gas-price', help="Set a static gas price (in GWEI)", type=GWEI)
 option_gas_strategy = click.option('--gas-strategy', help="Operate with a specified gas price strategy", type=click.STRING)  # TODO: GAS_STRATEGY_CHOICES
+option_key_material = click.option('--key-material', help="A pre-secured hex-encoded secret to use for private key derivations", type=click.STRING)
 option_max_gas_price = click.option('--max-gas-price', help="Maximum acceptable gas price (in GWEI)", type=GWEI)
 option_hw_wallet = click.option('--hw-wallet/--no-hw-wallet')
 option_light = click.option('--light', help="Indicate that node is light", is_flag=True, default=None)
 option_lonely = click.option('--lonely', help="Do not connect to seednodes", is_flag=True)
-option_m = click.option('--m', help="M-Threshold KFrags", type=click.INT)
-option_min_stake = click.option('--min-stake', help="The minimum stake the teacher must have to be locally accepted.", type=STAKED_TOKENS_RANGE, default=MIN_ALLOWED_LOCKED_TOKENS)
-option_n = click.option('--n', help="N-Total KFrags", type=click.INT)
+option_min_stake = click.option('--min-stake', help="The minimum stake the teacher must have to be locally accepted.", type=STAKED_TOKENS_RANGE, default=MIN_AUTHORIZATION)
+option_operator_address = click.option('--operator-address', help="Address to bond as an operator", type=EIP55_CHECKSUM_ADDRESS, required=True)
 option_parameters = click.option('--parameters', help="Filepath to a JSON file containing additional parameters", type=EXISTING_READABLE_FILE)
 option_participant_address = click.option('--participant-address', help="Participant's checksum address.", type=EIP55_CHECKSUM_ADDRESS)
+option_payment_provider = click.option('--payment-provider', 'payment_provider', help="Connection URL for payment method", type=click.STRING, required=False)
+option_payment_network = click.option('--payment-network', help="Payment network name", type=click.STRING, required=False)  # TODO: Choices
+option_payment_method = click.option('--payment-method', help="Payment method name", type=PAYMENT_METHOD_CHOICES, required=False)
 option_poa = click.option('--poa/--disable-poa', help="Inject POA middleware", is_flag=True, default=None)
 option_registry_filepath = click.option('--registry-filepath', help="Custom contract registry filepath", type=EXISTING_READABLE_FILE)
+option_policy_registry_filepath = click.option('--policy-registry-filepath', help="Custom contract registry filepath for policies", type=EXISTING_READABLE_FILE)
+option_shares = click.option('--shares', '-n', help="N-Total shares", type=click.INT)
 option_signer_uri = click.option('--signer', 'signer_uri', '-S', default=None, type=str)
-option_staking_address = click.option('--staking-address', help="Address of a NuCypher staker", type=EIP55_CHECKSUM_ADDRESS)
+option_staking_provider = click.option('--staking-provider', help="Staking provider ethereum address", type=EIP55_CHECKSUM_ADDRESS, required=True)
 option_teacher_uri = click.option('--teacher', 'teacher_uri', help="An Ursula URI to start learning from (seednode)", type=click.STRING)
+option_threshold = click.option('--threshold', '-m', help="M-Threshold KFrags", type=click.INT)
+option_treasure_map = click.option('--treasure-map', 'treasure_map', help="Encrypted treasure map as base64 for retrieval", type=click.STRING, required=True)
 _option_middleware = click.option('-Z', '--mock-networking', help="Use in-memory transport instead of networking", count=True)
 
 # Avoid circular input
@@ -72,6 +80,15 @@ option_rate = click.option('--rate', help="Policy rate per period (in wei)", typ
 # Alphabetical
 #
 
+def option_alice_verifying_key(required: bool = False):
+    return click.option(
+        '--alice-verifying-key',
+        '-avk',
+        help="Alice's verifying key as a hexadecimal string",
+        type=click.STRING,
+        required=required)
+
+
 def option_contract_name(required: bool = False):
     return click.option(
         '--contract-name',
@@ -84,7 +101,7 @@ def option_contract_name(required: bool = False):
 def option_controller_port(default=None):
     return click.option(
         '--controller-port',
-        help="The host port to run Alice HTTP services on",
+        help="The host port to run HTTP services on",
         type=NETWORK_PORT,
         default=default)
 
@@ -105,11 +122,12 @@ def option_label(required: bool = False):
         required=required)
 
 
-def option_message_kit(required: bool = False):
+def option_message_kit(required: bool = False, multiple: bool = False):
     return click.option(
         '--message-kit',
-        help="The message kit unicode string encoded in base64",
+        help='The message kit unicode string encoded in base64',
         type=click.STRING,
+        multiple=multiple,
         required=required)
 
 
@@ -132,9 +150,9 @@ def option_policy_encrypting_key(required: bool = False):
         required=required)
 
 
-def option_provider_uri(default=None, required: bool = False):
+def option_eth_provider_uri(default=None, required: bool = False):
     return click.option(
-        '--provider', 'provider_uri',
+        '--eth-provider', 'eth_provider_uri',
         help="Blockchain provider's URI i.e. 'file:///path/to/geth.ipc'",
         type=click.STRING,
         required=required,
