@@ -15,12 +15,18 @@ You should have received a copy of the GNU Affero General Public License
 along with nucypher.  If not, see <https://www.gnu.org/licenses/>.
 """
 
+
 import click
 import maya
+from constant_sorrow.constants import NO_KEYSTORE_ATTACHED
 
 from nucypher.blockchain.eth.sol.__conf__ import SOLIDITY_COMPILER_VERSION
 from nucypher.characters.banners import NUCYPHER_BANNER
-from nucypher.config.constants import DEFAULT_CONFIG_ROOT, USER_LOG_DIR, END_OF_POLICIES_PROBATIONARY_PERIOD
+from nucypher.config.constants import (
+    DEFAULT_CONFIG_ROOT,
+    USER_LOG_DIR,
+    END_OF_POLICIES_PROBATIONARY_PERIOD
+)
 
 
 def echo_version(ctx, param, value):
@@ -40,22 +46,36 @@ def echo_solidity_version(ctx, param, value):
 def echo_config_root_path(ctx, param, value):
     if not value or ctx.resilient_parsing:
         return
-    click.secho(DEFAULT_CONFIG_ROOT)
+    click.secho(str(DEFAULT_CONFIG_ROOT.absolute()))
     ctx.exit()
 
 
 def echo_logging_root_path(ctx, param, value):
     if not value or ctx.resilient_parsing:
         return
-    click.secho(USER_LOG_DIR)
+    click.secho(str(USER_LOG_DIR.absolute()))
     ctx.exit()
 
 
 def paint_new_installation_help(emitter, new_configuration, filepath):
     character_config_class = new_configuration.__class__
     character_name = character_config_class.NAME.lower()
+    if new_configuration.keystore != NO_KEYSTORE_ATTACHED:
+        maybe_public_key = new_configuration.keystore.id
+    else:
+        maybe_public_key = "(no keystore attached)"
+    emitter.message(f"Generated keystore", color='green')
+    emitter.message(f"""
+    
+Public Key:   {maybe_public_key}
+Path to Keystore: {new_configuration.keystore_dir}
 
-    emitter.message(f"Generated keyring {new_configuration.keyring_root}", color='green')
+- You can share your public key with anyone. Others need it to interact with you.
+- Never share secret keys with anyone! 
+- Backup your keystore! Character keys are required to interact with the protocol!
+- Remember your password! Without the password, it's impossible to decrypt the key!
+
+""")
 
     default_config_filepath = True
     if new_configuration.default_filepath() != filepath:
@@ -68,14 +88,8 @@ def paint_new_installation_help(emitter, new_configuration, filepath):
         emitter.message(f'* NOTE: for a non-default configuration filepath use `--config-file "{filepath}"` '
                         f'with subsequent `{character_name}` CLI commands', color='yellow')
 
-    # Felix
-    if character_name == 'felix':
-        hint = '''
-To initialize a new faucet recipient database run: nucypher felix createdb 
-'''
-
     # Ursula
-    elif character_name == 'ursula':
+    if character_name == 'ursula':
         hint = '''
 * Review configuration  -> nucypher ursula config
 * Start working         -> nucypher ursula run
@@ -101,6 +115,7 @@ To initialize a new faucet recipient database run: nucypher felix createdb
 
     else:
         raise ValueError(f'Unknown character type "{character_name}"')
+
     emitter.echo(hint, color='green')
 
 
