@@ -17,24 +17,22 @@
 
 
 import json
-import os
 import re
 from abc import ABC, abstractmethod
 from decimal import Decimal
+from pathlib import Path
 from tempfile import TemporaryDirectory
-from typing import Union, Callable, Optional, List
+from typing import Callable, List, Optional, Union
 
 from constant_sorrow.constants import (
-    UNKNOWN_VERSION,
-    UNINITIALIZED_CONFIGURATION,
-    NO_KEYRING_ATTACHED,
-    NO_BLOCKCHAIN_CONNECTION,
-    FEDERATED_ADDRESS,
     DEVELOPMENT_CONFIGURATION,
-    LIVE_CONFIGURATION
+    LIVE_CONFIGURATION,
+    NO_BLOCKCHAIN_CONNECTION,
+    NO_KEYSTORE_ATTACHED,
+    UNINITIALIZED_CONFIGURATION,
+    UNKNOWN_VERSION
 )
 from eth_utils.address import is_checksum_address
-from umbral.signing import Signature
 
 from nucypher.blockchain.eth.interfaces import BlockchainInterfaceFactory
 from nucypher.blockchain.eth.networks import NetworksInventory
@@ -46,14 +44,16 @@ from nucypher.blockchain.eth.registry import (
 from nucypher.blockchain.eth.signers import Signer
 from nucypher.characters.lawful import Ursula
 from nucypher.config import constants
-from nucypher.config.keyring import NucypherKeyring
 from nucypher.config.storages import (
     ForgetfulNodeStorage,
     LocalFileBasedNodeStorage,
     NodeStorage
 )
+from nucypher.config.util import cast_paths_from
+from nucypher.crypto.keystore import Keystore
 from nucypher.crypto.powers import CryptoPower, CryptoPowerUp
 from nucypher.network.middleware import RestMiddleware
+from nucypher.policy.payment import PAYMENT_METHODS
 from nucypher.utilities.logging import Logger
 
 
@@ -133,8 +133,8 @@ class BaseConfiguration(ABC):
         pass
 
     def __init__(self,
-                 config_root: str = None,
-                 filepath: str = None,
+                 config_root: Optional[Path] = None,
+                 filepath: Optional[Path] = None,
                  *args, **kwargs):
 
         if self.NAME is NotImplemented:
@@ -143,7 +143,7 @@ class BaseConfiguration(ABC):
 
         self.config_root = config_root or self.DEFAULT_CONFIG_ROOT
         if not filepath:
-            filepath = os.path.join(self.config_root, self.generate_filename())
+            filepath = self.config_root / self.generate_filename()
         self.filepath = filepath
 
         super().__init__()
@@ -188,17 +188,17 @@ class BaseConfiguration(ABC):
         return filename
 
     @classmethod
-    def default_filepath(cls, config_root: str = None) -> str:
+    def default_filepath(cls, config_root: Optional[Path] = None) -> Path:
         """
         Generates the default configuration filepath for the class.
 
         :return: The generated filepath string
         """
         filename = cls.generate_filename()
-        default_path = os.path.join(config_root or cls.DEFAULT_CONFIG_ROOT, filename)
+        default_path = (config_root or cls.DEFAULT_CONFIG_ROOT) / filename
         return default_path
 
-    def generate_filepath(self, filepath: str = None, modifier: str = None, override: bool = False) -> str:
+    def generate_filepath(self, filepath: Optional[Path] = None, modifier: str = None, override: bool = False) -> Path:
         """
         Generates a filepath for saving to writing to a configuration file.
 
@@ -217,12 +217,12 @@ class BaseConfiguration(ABC):
         """
         if not filepath:
             filename = self.generate_filename()
-            filepath = os.path.join(self.config_root, filename)
-        if os.path.exists(filepath) and not override:
+            filepath = self.config_root / filename
+        if filepath.exists() and not override:
             if not modifier:
                 raise FileExistsError(f"{filepath} exists and no filename modifier supplied.")
             filename = self.generate_filename(modifier=modifier)
-            filepath = os.path.join(self.config_root, filename)
+            filepath = self.config_root / filename
         self.filepath = filepath
         return filepath
 
@@ -233,14 +233,14 @@ class BaseConfiguration(ABC):
 
         :return: None.
         """
-        if not os.path.exists(self.config_root):
+        if not self.config_root.exists():
             try:
-                os.mkdir(self.config_root, mode=0o755)
+                self.config_root.mkdir(mode=0o755)
             except FileNotFoundError:
-                os.makedirs(self.config_root, mode=0o755)
+                self.config_root.mkdir(parents=True, mode=0o755)
 
     @classmethod
-    def peek(cls, filepath: str, field: str) -> Union[str, None]:
+    def peek(cls, filepath: Path, field: str) -> Union[str, None]:
         payload = cls._read_configuration_file(filepath=filepath)
         try:
             result = payload[field]
@@ -248,30 +248,30 @@ class BaseConfiguration(ABC):
             raise cls.ConfigurationError(f"Cannot peek; No such configuration field '{field}', options are {list(payload.keys())}")
         return result
 
-    def to_configuration_file(self, filepath: str = None, modifier: str = None, override: bool = False) -> str:
+    def to_configuration_file(self, filepath: Optional[Path] = None, modifier: str = None, override: bool = False) -> Path:
         filepath = self.generate_filepath(filepath=filepath, modifier=modifier, override=override)
         self._ensure_config_root_exists()
         filepath = self._write_configuration_file(filepath=filepath, override=override)
         return filepath
 
     @classmethod
-    def from_configuration_file(cls, filepath: str = None, **overrides) -> 'BaseConfiguration':
+    def from_configuration_file(cls, filepath: Optional[Path] = None, **overrides) -> 'BaseConfiguration':
         filepath = filepath or cls.default_filepath()
         payload = cls._read_configuration_file(filepath=filepath)
         instance = cls(filepath=filepath, **payload, **overrides)
         return instance
 
     @classmethod
-    def _read_configuration_file(cls, filepath: str) -> dict:
+    def _read_configuration_file(cls, filepath: Path) -> dict:
         """Reads `filepath` and returns the deserialized JSON payload dict."""
         with open(filepath, 'r') as file:
             raw_contents = file.read()
-            payload = cls.deserialize(raw_contents, payload_label=filepath)
+            payload = cls.deserialize(raw_contents, payload_label=str(filepath))
         return payload
 
-    def _write_configuration_file(self, filepath: str, override: bool = False) -> str:
+    def _write_configuration_file(self, filepath: Path, override: bool = False) -> Path:
         """Writes to `filepath` and returns the written filepath.  Raises `FileExistsError` if the file exists."""
-        if os.path.exists(str(filepath)) and not override:
+        if filepath.exists() and not override:
             raise FileExistsError(f"{filepath} exists and no filename modifier supplied.")
         with open(filepath, 'w') as file:
             file.write(self.serialize())
@@ -279,7 +279,15 @@ class BaseConfiguration(ABC):
 
     def serialize(self, serializer=json.dumps) -> str:
         """Returns the JSON serialized output of `static_payload`"""
+        def _stringify_paths(d: dict):
+            for key, value in d.items():
+                if isinstance(value, Path):
+                    d[key] = str(value)
+                if isinstance(value, dict):
+                    _stringify_paths(value)
+
         payload = self.static_payload()
+        _stringify_paths(payload)
         payload['version'] = self.VERSION
         serialized_payload = serializer(payload, indent=self.INDENTATION)
         return serialized_payload
@@ -291,11 +299,13 @@ class BaseConfiguration(ABC):
         version = deserialized_payload.pop('version', UNKNOWN_VERSION)
         if version != cls.VERSION:
             label = f"'{payload_label}' " if payload_label else ""
-            raise cls.OldVersion(f"Configuration {label}is the wrong version "
+            raise cls.OldVersion(f"Configuration {label} is the wrong version "
                                  f"Expected version {cls.VERSION}; Got version {version}")
+
+        deserialized_payload = cast_paths_from(cls, deserialized_payload)
         return deserialized_payload
 
-    def update(self, filepath: str = None, **updates) -> None:
+    def update(self, filepath: Optional[Path] = None, **updates) -> None:
         for field, value in updates.items():
             try:
                 getattr(self, field)
@@ -312,21 +322,28 @@ class CharacterConfiguration(BaseConfiguration):
     'Sideways Engagement' of Character classes; a reflection of input parameters.
     """
 
-    VERSION = 2  # bump when static payload scheme changes
+    VERSION = 4  # bump when static payload scheme changes
 
     CHARACTER_CLASS = NotImplemented
     DEFAULT_CONTROLLER_PORT = NotImplemented
+    MNEMONIC_KEYSTORE = False
     DEFAULT_DOMAIN = NetworksInventory.DEFAULT
     DEFAULT_NETWORK_MIDDLEWARE = RestMiddleware
     TEMP_CONFIGURATION_DIR_PREFIX = 'tmp-nucypher'
     SIGNER_ENVVAR = None
 
-    # When we begin to support other threshold schemes, this will be one of the concepts that makes us want a factory.  #571
+    # When we begin to support other threshold schemes,
+    # this will be one of the concepts that makes us want a factory.  #571
     known_node_class = Ursula
 
     # Gas
     DEFAULT_GAS_STRATEGY = 'fast'
 
+    # Payments
+    DEFAULT_PAYMENT_METHOD = 'SubscriptionManager'
+    DEFAULT_PAYMENT_NETWORK = 'polygon'
+    DEFAULT_FEDERATED_PAYMENT_METHOD = 'Free'
+
     # Fields specified here are *not* passed into the Character's constructor
     # and can be understood as configuration fields only.
     _CONFIG_FIELDS = ('config_root',
@@ -336,15 +353,17 @@ class CharacterConfiguration(BaseConfiguration):
                       'gas_strategy',
                       'max_gas_price',  # gwei
                       'signer_uri',
-                      'keyring_root'
+                      'keystore_path',
+                      'payment_provider',
+                      'payment_network'
                       )
 
     def __init__(self,
 
                  # Base
                  emitter=None,
-                 config_root: str = None,
-                 filepath: str = None,
+                 config_root: Optional[Path] = None,
+                 filepath: Optional[Path] = None,
 
                  # Mode
                  dev_mode: bool = False,
@@ -354,9 +373,9 @@ class CharacterConfiguration(BaseConfiguration):
                  checksum_address: str = None,
                  crypto_power: CryptoPower = None,
 
-                 # Keyring
-                 keyring: NucypherKeyring = None,
-                 keyring_root: str = None,
+                 # Keystore
+                 keystore: Keystore = None,
+                 keystore_path: Optional[Path] = None,
 
                  # Learner
                  learn_on_same_thread: bool = False,
@@ -366,7 +385,6 @@ class CharacterConfiguration(BaseConfiguration):
                  # Network
                  controller_port: int = None,
                  domain: str = DEFAULT_DOMAIN,
-                 interface_signature: Signature = None,
                  network_middleware: RestMiddleware = None,
                  lonely: bool = False,
 
@@ -379,16 +397,24 @@ class CharacterConfiguration(BaseConfiguration):
                  # Blockchain
                  poa: bool = None,
                  light: bool = False,
-                 provider_uri: str = None,
+                 eth_provider_uri: str = None,
                  gas_strategy: Union[Callable, str] = DEFAULT_GAS_STRATEGY,
                  max_gas_price: Optional[int] = None,
                  signer_uri: str = None,
 
-                 # Registry
-                 registry: BaseContractRegistry = None,
-                 registry_filepath: str = None,
+                 # Payments
+                 # TODO: Resolve code prefixing below, possibly with the use of nested configuration fields
+                 payment_method: str = None,
+                 payment_provider: str = None,
+                 payment_network: str = None,
 
-                 # Deployed Workers
+                 # Registries
+                 registry: BaseContractRegistry = None,
+                 registry_filepath: Optional[Path] = None,
+                 policy_registry: BaseContractRegistry = None,
+                 policy_registry_filepath: Optional[Path] = None,
+
+                 # Deployed Operators
                  worker_data: dict = None
                  ):
 
@@ -402,25 +428,31 @@ class CharacterConfiguration(BaseConfiguration):
         self.is_me = True
         self.checksum_address = checksum_address
 
-        # Keyring
+        # Keystore
         self.crypto_power = crypto_power
-        self.keyring = keyring or NO_KEYRING_ATTACHED
-        self.keyring_root = keyring_root or UNINITIALIZED_CONFIGURATION
+        if keystore_path and not keystore:
+            keystore = Keystore(keystore_path=keystore_path)
+        self.__keystore = self.__keystore = keystore or NO_KEYSTORE_ATTACHED.bool_value(False)
+        self.keystore_dir = Path(keystore.keystore_path).parent if keystore else UNINITIALIZED_CONFIGURATION
 
         # Contract Registry
         if registry and registry_filepath:
             if registry.filepath != registry_filepath:
-                error = f"Inconsistent registry filepaths for '{registry.filepath}' and '{registry_filepath}'."
+                error = f"Inconsistent registry filepaths for '{registry.filepath.absolute()}'" \
+                        f" and '{registry_filepath.absolute()}'."
                 raise ValueError(error)
             else:
                 self.log.warn(f"Registry and registry filepath were both passed.")
         self.registry = registry or NO_BLOCKCHAIN_CONNECTION.bool_value(False)
         self.registry_filepath = registry_filepath or UNINITIALIZED_CONFIGURATION
 
+        self.policy_registry = policy_registry or NO_BLOCKCHAIN_CONNECTION.bool_value(False)
+        self.policy_registry_filepath = policy_registry_filepath or UNINITIALIZED_CONFIGURATION
+
         # Blockchain
         self.poa = poa
         self.is_light = light
-        self.provider_uri = provider_uri or NO_BLOCKCHAIN_CONNECTION
+        self.eth_provider_uri = eth_provider_uri or NO_BLOCKCHAIN_CONNECTION
         self.signer_uri = signer_uri or None
 
         # Learner
@@ -439,7 +471,7 @@ class CharacterConfiguration(BaseConfiguration):
         self.config_file_location = filepath or UNINITIALIZED_CONFIGURATION
         self.config_root = UNINITIALIZED_CONFIGURATION
 
-        # Deployed Workers
+        # Deployed Operators
         self.worker_data = worker_data
 
         #
@@ -454,7 +486,7 @@ class CharacterConfiguration(BaseConfiguration):
             # Check for incompatible values
             blockchain_args = {'filepath': registry_filepath,
                                'poa': poa,
-                               'provider_uri': provider_uri,
+                               'eth_provider_uri': eth_provider_uri,
                                'gas_strategy': gas_strategy,
                                'max_gas_price': max_gas_price}
             if any(blockchain_args.values()):
@@ -466,11 +498,17 @@ class CharacterConfiguration(BaseConfiguration):
                 # federated configuration.
                 self.poa = False
                 self.is_light = False
-                self.provider_uri = None
+                self.eth_provider_uri = None
                 self.registry_filepath = None
+                self.policy_registry_filepath = None
                 self.gas_strategy = None
                 self.max_gas_price = None
 
+            # Federated Payments
+            self.payment_method = payment_method or self.DEFAULT_FEDERATED_PAYMENT_METHOD
+            self.payment_network = payment_network
+            self.payment_provider = payment_provider
+
         #
         # Decentralized
         #
@@ -478,16 +516,16 @@ class CharacterConfiguration(BaseConfiguration):
         else:
             self.gas_strategy = gas_strategy
             self.max_gas_price = max_gas_price  # gwei
-            is_initialized = BlockchainInterfaceFactory.is_interface_initialized(provider_uri=self.provider_uri)
-            if not is_initialized and provider_uri:
-                BlockchainInterfaceFactory.initialize_interface(provider_uri=self.provider_uri,
+            is_initialized = BlockchainInterfaceFactory.is_interface_initialized(eth_provider_uri=self.eth_provider_uri)
+            if not is_initialized and eth_provider_uri:
+                BlockchainInterfaceFactory.initialize_interface(eth_provider_uri=self.eth_provider_uri,
                                                                 poa=self.poa,
                                                                 light=self.is_light,
                                                                 emitter=emitter,
                                                                 gas_strategy=self.gas_strategy,
                                                                 max_gas_price=self.max_gas_price)
             else:
-                self.log.warn(f"Using existing blockchain interface connection ({self.provider_uri}).")
+                self.log.warn(f"Using existing blockchain interface connection ({self.eth_provider_uri}).")
 
             if not self.registry:
                 # TODO: These two code blocks are untested.
@@ -501,6 +539,28 @@ class CharacterConfiguration(BaseConfiguration):
             self.testnet = self.domain != NetworksInventory.MAINNET
             self.signer = Signer.from_signer_uri(self.signer_uri, testnet=self.testnet)
 
+            #
+            # Onchain Payments & Policies
+            #
+
+            # FIXME: Enforce this for Ursula/Alice but not Bob?
+            from nucypher.config.characters import BobConfiguration
+            if not isinstance(self, BobConfiguration):
+                # if not payment_provider:
+                #     raise self.ConfigurationError("payment provider is required.")
+                self.payment_method = payment_method or self.DEFAULT_PAYMENT_METHOD
+                self.payment_network = payment_network or self.DEFAULT_PAYMENT_NETWORK
+                self.payment_provider = payment_provider or (self.eth_provider_uri or None)  # default to L1 payments
+
+                # TODO: Dedupe
+                if not self.policy_registry:
+                    if not self.policy_registry_filepath:
+                        self.log.info(f"Fetching latest policy registry from source.")
+                        self.policy_registry = InMemoryContractRegistry.from_latest_publication(network=self.payment_network)
+                    else:
+                        self.policy_registry = LocalContractRegistry(filepath=self.policy_registry_filepath)
+                        self.log.info(f"Using local policy registry ({self.policy_registry}).")
+
         if dev_mode:
             self.__temp_dir = UNINITIALIZED_CONFIGURATION
             self._setup_node_storage()
@@ -514,23 +574,29 @@ class CharacterConfiguration(BaseConfiguration):
         # Network
         self.controller_port = controller_port or self.DEFAULT_CONTROLLER_PORT
         self.network_middleware = network_middleware or self.DEFAULT_NETWORK_MIDDLEWARE(registry=self.registry)
-        self.interface_signature = interface_signature
-
+        
         super().__init__(filepath=self.config_file_location, config_root=self.config_root)
 
     def __call__(self, **character_kwargs):
         return self.produce(**character_kwargs)
 
+    @property
+    def keystore(self) -> Keystore:
+        return self.__keystore
+
+    def attach_keystore(self, keystore: Keystore) -> None:
+        self.__keystore = keystore
+
     @classmethod
-    def checksum_address_from_filepath(cls, filepath: str) -> str:
+    def checksum_address_from_filepath(cls, filepath: Path) -> str:
         pattern = re.compile(r'''
                              (^\w+)-
-                             (0x{1}         # Then, 0x the start of the string, exactly once
+                             (0x{1}           # Then, 0x the start of the string, exactly once
                              [0-9a-fA-F]{40}) # Followed by exactly 40 hex chars
                              ''',
                              re.VERBOSE)
 
-        filename = os.path.basename(filepath)
+        filename = filepath.name
         match = pattern.match(filename)
 
         if match:
@@ -557,10 +623,10 @@ class CharacterConfiguration(BaseConfiguration):
         return super().update(filepath=self.config_file_location, **kwargs)
 
     @classmethod
-    def generate(cls, password: str, *args, **kwargs):
+    def generate(cls, password: str, key_material: Optional[bytes] = None, *args, **kwargs):
         """Shortcut: Hook-up a new initial installation and configuration."""
         node_config = cls(dev_mode=False, *args, **kwargs)
-        node_config.initialize(password=password)
+        node_config.initialize(key_material=key_material, password=password)
         return node_config
 
     def cleanup(self) -> None:
@@ -572,12 +638,21 @@ class CharacterConfiguration(BaseConfiguration):
         return self.__dev_mode
 
     def _setup_node_storage(self, node_storage=None) -> None:
-        if self.dev_mode:
-            node_storage = ForgetfulNodeStorage(registry=self.registry, federated_only=self.federated_only)
-        elif not node_storage:
-            node_storage = LocalFileBasedNodeStorage(registry=self.registry,
-                                                     config_root=self.config_root,
-                                                     federated_only=self.federated_only)
+        # TODO: Disables node metadata persistence..
+        # if self.dev_mode:
+        #     node_storage = ForgetfulNodeStorage(registry=self.registry, federated_only=self.federated_only)
+
+        # TODO: Forcibly clears the filesystem of any stored node metadata and certificates...
+        local_node_storage = LocalFileBasedNodeStorage(
+            registry=self.registry,
+            config_root=self.config_root,
+            federated_only=self.federated_only
+        )
+        local_node_storage.clear()
+        self.log.info(f'Cleared peer metadata from {local_node_storage.root_dir}')
+
+        # TODO: Always sets up nodes for in-memory node metadata storage
+        node_storage = ForgetfulNodeStorage(registry=self.registry, federated_only=self.federated_only)
         self.node_storage = node_storage
 
     def forget_nodes(self) -> None:
@@ -587,9 +662,7 @@ class CharacterConfiguration(BaseConfiguration):
 
     def destroy(self) -> None:
         """Parse a node configuration and remove all associated files from the filesystem"""
-        self.attach_keyring()
-        self.keyring.destroy()
-        os.remove(self.config_file_location)
+        self.config_file_location.unlink()
 
     def generate_parameters(self, **overrides) -> dict:
         """
@@ -606,20 +679,21 @@ class CharacterConfiguration(BaseConfiguration):
         return character
 
     @classmethod
-    def assemble(cls, filepath: str = None, **overrides) -> dict:
+    def assemble(cls, filepath: Optional[Path] = None, **overrides) -> dict:
         """
         Warning: This method allows mutation and may result in an inconsistent configuration.
         """
         payload = cls._read_configuration_file(filepath=filepath)
         node_storage = cls.load_node_storage(storage_payload=payload['node_storage'],
                                              federated_only=payload['federated_only'])
-        domain = payload['domain']
         max_gas_price = payload.get('max_gas_price')  # gwei
         if max_gas_price:
             max_gas_price = Decimal(max_gas_price)
 
         # Assemble
-        payload.update(dict(node_storage=node_storage, domain=domain, max_gas_price=max_gas_price))
+        payload.update(dict(node_storage=node_storage, max_gas_price=max_gas_price))
+        payload = cast_paths_from(cls, payload)
+
         # Filter out None values from **overrides to detect, well, overrides...
         # Acts as a shim for optional CLI flags.
         overrides = {k: v for k, v in overrides.items() if v is not None}
@@ -628,7 +702,7 @@ class CharacterConfiguration(BaseConfiguration):
 
     @classmethod
     def from_configuration_file(cls,
-                                filepath: str = None,
+                                filepath: Optional[Path] = None,
                                 **overrides  # < ---- Inlet for CLI Flags
                                 ) -> 'CharacterConfiguration':
         """Initialize a CharacterConfiguration from a JSON file."""
@@ -640,13 +714,13 @@ class CharacterConfiguration(BaseConfiguration):
     def validate(self) -> bool:
 
         # Top-level
-        if not os.path.exists(self.config_root):
+        if not self.config_root.exists():
             raise self.ConfigurationError(f'No configuration directory found at {self.config_root}.')
 
         # Sub-paths
         filepaths = self.runtime_filepaths
         for field, path in filepaths.items():
-            if path and not os.path.exists(path):
+            if path and not path.exists():
                 message = 'Missing configuration file or directory: {}.'
                 if 'registry' in path:
                     message += ' Did you mean to pass --federated-only?'
@@ -654,14 +728,14 @@ class CharacterConfiguration(BaseConfiguration):
         return True
 
     def static_payload(self) -> dict:
-        """Exported static configuration values for initializing Ursula"""
-
+        """JSON-Exported static configuration values for initializing Ursula"""
+        keystore_path = str(self.keystore.keystore_path) if self.keystore else None
         payload = dict(
 
             # Identity
             federated_only=self.federated_only,
             checksum_address=self.checksum_address,
-            keyring_root=self.keyring_root,
+            keystore_path=keystore_path,
 
             # Behavior
             domain=self.domain,
@@ -675,10 +749,10 @@ class CharacterConfiguration(BaseConfiguration):
 
         # Optional values (mode)
         if not self.federated_only:
-            if self.provider_uri:
+            if self.eth_provider_uri:
                 if not self.signer_uri:
-                    self.signer_uri = self.provider_uri
-                payload.update(dict(provider_uri=self.provider_uri,
+                    self.signer_uri = self.eth_provider_uri
+                payload.update(dict(eth_provider_uri=self.eth_provider_uri,
                                     poa=self.poa,
                                     light=self.is_light,
                                     signer_uri=self.signer_uri))
@@ -695,9 +769,13 @@ class CharacterConfiguration(BaseConfiguration):
 
         return payload
 
-    @property  # TODO: Graduate to a method and "derive" dynamic from static payload.
+    @property
     def dynamic_payload(self) -> dict:
-        """Exported dynamic configuration values for initializing Ursula"""
+        """
+        Exported dynamic configuration values for initializing Ursula.
+        These values are used to init a character instance but are *not*
+        saved to the JSON configuration.
+        """
         payload = dict()
         if not self.federated_only:
             payload.update(dict(registry=self.registry, signer=self.signer))
@@ -705,12 +783,12 @@ class CharacterConfiguration(BaseConfiguration):
         payload.update(dict(network_middleware=self.network_middleware or self.DEFAULT_NETWORK_MIDDLEWARE(),
                             known_nodes=self.known_nodes,
                             node_storage=self.node_storage,
-                            keyring=self.keyring,
+                            keystore=self.keystore,
                             crypto_power_ups=self.derive_node_power_ups()))
 
         return payload
 
-    def generate_filepath(self, filepath: str = None, modifier: str = None, override: bool = False) -> str:
+    def generate_filepath(self, filepath: Optional[Path] = None, modifier: str = None, override: bool = False) -> Path:
         modifier = modifier or self.checksum_address
         filepath = super().generate_filepath(filepath=filepath, modifier=modifier, override=override)
         return filepath
@@ -718,16 +796,16 @@ class CharacterConfiguration(BaseConfiguration):
     @property
     def runtime_filepaths(self) -> dict:
         filepaths = dict(config_root=self.config_root,
-                         keyring_root=self.keyring_root,
+                         keystore_dir=self.keystore_dir,
                          registry_filepath=self.registry_filepath)
         return filepaths
 
     @classmethod
-    def generate_runtime_filepaths(cls, config_root: str) -> dict:
+    def generate_runtime_filepaths(cls, config_root: Path) -> dict:
         """Dynamically generate paths based on configuration root directory"""
         filepaths = dict(config_root=config_root,
-                         config_file_location=os.path.join(config_root, cls.generate_filename()),
-                         keyring_root=os.path.join(config_root, 'keyring'))
+                         config_file_location=config_root / cls.generate_filename(),
+                         keystore_dir=config_root / 'keystore')
         return filepaths
 
     def _cache_runtime_filepaths(self) -> None:
@@ -737,36 +815,28 @@ class CharacterConfiguration(BaseConfiguration):
             if getattr(self, field) is UNINITIALIZED_CONFIGURATION:
                 setattr(self, field, filepath)
 
-    def attach_keyring(self, checksum_address: str = None, *args, **kwargs) -> None:
-        account = checksum_address or self.checksum_address
-        if not account:
-            raise self.ConfigurationError("No account specified to unlock keyring")
-        if self.keyring is not NO_KEYRING_ATTACHED:
-            if self.keyring.checksum_address != account:
-                raise self.ConfigurationError("There is already a keyring attached to this configuration.")
-            return
-        self.keyring = NucypherKeyring(keyring_root=self.keyring_root, account=account, *args, **kwargs)
-
     def derive_node_power_ups(self) -> List[CryptoPowerUp]:
         power_ups = list()
         if self.is_me and not self.dev_mode:
             for power_class in self.CHARACTER_CLASS._default_crypto_powerups:
-                power_up = self.keyring.derive_crypto_power(power_class)
+                power_up = self.keystore.derive_crypto_power(power_class)
                 power_ups.append(power_up)
         return power_ups
 
-    def initialize(self, password: str) -> str:
+    def initialize(self, password: str, key_material: Optional[bytes] = None) -> str:
         """Initialize a new configuration and write installation files to disk."""
 
         # Development
         if self.dev_mode:
             self.__temp_dir = TemporaryDirectory(prefix=self.TEMP_CONFIGURATION_DIR_PREFIX)
-            self.config_root = self.__temp_dir.name
+            self.config_root = Path(self.__temp_dir.name)
 
         # Persistent
         else:
             self._ensure_config_root_exists()
-            self.write_keyring(password=password)
+            self.write_keystore(key_material=key_material,
+                                password=password,
+                                interactive=self.MNEMONIC_KEYSTORE)
 
         self._cache_runtime_filepaths()
         self.node_storage.initialize()
@@ -778,29 +848,24 @@ class CharacterConfiguration(BaseConfiguration):
         # Success
         message = "Created nucypher installation files at {}".format(self.config_root)
         self.log.debug(message)
-        return self.config_root
+        return Path(self.config_root)
 
-    def write_keyring(self, password: str, checksum_address: str = None, **generation_kwargs) -> NucypherKeyring:
+    def write_keystore(self, password: str, key_material: Optional[bytes] = None, interactive: bool = True) -> Keystore:
+        if key_material:
+            self.__keystore = Keystore.import_secure(key_material=key_material,
+                                                     password=password,
+                                                     keystore_dir=self.keystore_dir)
+        else:
+            if interactive:
+                self.__keystore = Keystore.generate(password=password,
+                                                    keystore_dir=self.keystore_dir,
+                                                    interactive=interactive)
+            else:
+                self.__keystore, _ = Keystore.generate(password=password,
+                                                       keystore_dir=self.keystore_dir,
+                                                       interactive=interactive)
 
-        # Configure checksum address
-        checksum_address = checksum_address or self.checksum_address
-        if self.federated_only:
-            checksum_address = FEDERATED_ADDRESS
-        elif not checksum_address:
-            raise self.ConfigurationError(f'No checksum address provided for decentralized configuration.')
-
-        # Generate new keys
-        self.keyring = NucypherKeyring.generate(password=password,
-                                                keyring_root=self.keyring_root,
-                                                checksum_address=checksum_address,
-                                                **generation_kwargs)
-
-        # In the case of a federated keyring generation,
-        # the generated federated address must be set here.
-        if self.federated_only:
-            self.checksum_address = self.keyring.checksum_address
-
-        return self.keyring
+        return self.keystore
 
     @classmethod
     def load_node_storage(cls, storage_payload: dict, federated_only: bool):
@@ -810,3 +875,31 @@ class CharacterConfiguration(BaseConfiguration):
         storage_class = node_storage_subclasses[storage_type]
         node_storage = storage_class.from_payload(payload=storage_payload, federated_only=federated_only)
         return node_storage
+
+    def configure_payment_method(self):
+        # TODO: finalize config fields
+        # Strategy-Based (current implementation, inflexible & hardcoded)
+        # 'payment_strategy': 'SubscriptionManager'
+        # 'payment_network': 'matic'
+        # 'payment_provider': 'https:///matic.infura.io....'
+        # Contract-Targeted (alternative implementation, flexible & generic)
+        # 'payment': {
+        #     'contract': '0xdeadbeef'
+        #     'abi': '/home/abi/sm.json'
+        #     'function': 'isPolicyActive'
+        #     'provider': 'https:///matic.infura.io....'
+        # }
+
+        try:
+            payment_class = PAYMENT_METHODS[self.payment_method]
+        except KeyError:
+            raise KeyError(f'Unknown payment verifier "{self.payment_method}"')
+
+        if payment_class.ONCHAIN:
+            # on-chain payment strategies require a blockchain connection
+            payment_strategy = payment_class(network=self.payment_network,
+                                             eth_provider=self.payment_provider,
+                                             registry=self.policy_registry)
+        else:
+            payment_strategy = payment_class()
+        return payment_strategy
