@@ -16,25 +16,23 @@
 """
 
 
+import base64
+import hashlib
 import json
 from pathlib import Path
-from typing import Union, Optional, Dict, Callable
+from typing import Callable, Dict, Optional, Union
 
-import base64
 import constant_sorrow
-import hashlib
-import maya
-import os
-from bytestring_splitter import VariableLengthBytestring, BytestringKwargifier
+from bytestring_splitter import BytestringKwargifier, VariableLengthBytestring
 from constant_sorrow.constants import ALICE, BOB, NO_SIGNATURE
 from hexbytes.main import HexBytes
-from umbral.keys import UmbralPublicKey
+
+from nucypher_core.umbral import PublicKey
 
 from nucypher.characters.base import Character
 from nucypher.characters.lawful import Alice, Bob
 from nucypher.config.constants import DEFAULT_CONFIG_ROOT
-from nucypher.crypto.powers import SigningPower, DecryptingPower
-from nucypher.policy.collections import SignedTreasureMap
+from nucypher.crypto.powers import DecryptingPower, SigningPower
 
 
 class Card:
@@ -44,14 +42,14 @@ class Card:
 
     _alice_specification = dict(
         character_flag=(bytes, 8),
-        verifying_key=(UmbralPublicKey, 33),
+        verifying_key=(PublicKey, PublicKey.serialized_size()),
         nickname=(bytes, VariableLengthBytestring),
     )
 
     _bob_specification = dict(
         character_flag=(bytes, 8),
-        verifying_key=(UmbralPublicKey, 33),
-        encrypting_key=(UmbralPublicKey, 33),
+        verifying_key=(PublicKey, PublicKey.serialized_size()),
+        encrypting_key=(PublicKey, PublicKey.serialized_size()),
         nickname=(bytes, VariableLengthBytestring),
     )
 
@@ -82,8 +80,8 @@ class Card:
 
     def __init__(self,
                  character_flag: Union[ALICE, BOB],
-                 verifying_key: Union[UmbralPublicKey, bytes],
-                 encrypting_key: Optional[Union[UmbralPublicKey, bytes]] = None,
+                 verifying_key: Union[PublicKey, bytes],
+                 encrypting_key: Optional[Union[PublicKey, bytes]] = None,
                  nickname: Optional[Union[bytes, str]] = None):
 
         try:
@@ -93,16 +91,16 @@ class Card:
         self.__character_flag = character_flag
 
         if isinstance(verifying_key, bytes):
-            verifying_key = UmbralPublicKey.from_bytes(verifying_key)
+            verifying_key = PublicKey.from_bytes(verifying_key)
         self.__verifying_key = verifying_key    # signing public key
 
         if isinstance(encrypting_key, bytes):
-            encrypting_key = UmbralPublicKey.from_bytes(encrypting_key)
+            encrypting_key = PublicKey.from_bytes(encrypting_key)
         self.__encrypting_key = encrypting_key  # public key
 
         if isinstance(nickname, str):
             nickname = nickname.encode()
-        self.__nickname = nickname
+        self.__nickname = nickname or None
 
         self.__validate()
 
@@ -143,6 +141,8 @@ class Card:
         payload = self.__payload
         if self.nickname:
             payload += VariableLengthBytestring(self.__nickname)
+        else:
+            payload += VariableLengthBytestring(b'')
         return payload
 
     def __hex__(self) -> str:
@@ -253,11 +253,11 @@ class Card:
 
 
     @property
-    def verifying_key(self) -> UmbralPublicKey:
+    def verifying_key(self) -> PublicKey:
         return self.__verifying_key
 
     @property
-    def encrypting_key(self) -> UmbralPublicKey:
+    def encrypting_key(self) -> PublicKey:
         return self.__encrypting_key
 
     @property
@@ -270,6 +270,7 @@ class Card:
             return self.__nickname.decode()
 
     def set_nickname(self, nickname: str) -> None:
+        nickname = nickname.replace(' ', '_')
         if len(nickname.encode()) > self.__MAX_NICKNAME_SIZE:
             raise ValueError(f'New nickname exceeds maximum size ({self.__MAX_NICKNAME_SIZE} bytes)')
         self.__nickname = nickname.encode()
@@ -295,11 +296,11 @@ class Card:
         return exists
 
     def save(self, encoder: Callable = base64.b64encode, overwrite: bool = False) -> Path:
-        if not self.CARD_DIR.exists():
-            os.mkdir(str(self.CARD_DIR))
+        if not self.CARD_DIR.is_dir():
+            self.CARD_DIR.mkdir()
         if self.is_saved and not overwrite:
             raise FileExistsError('Card exists. Pass overwrite=True to allow this operation.')
-        with open(str(self.filepath), 'wb') as file:
+        with open(self.filepath, 'wb') as file:
             file.write(encoder(bytes(self)))
         return Path(self.filepath)
 
@@ -310,7 +311,7 @@ class Card:
             nickname, _id = identifier.split(cls.__DELIMITER)
         except ValueError:
             nickname = identifier
-        filenames = [f for f in os.listdir(Card.CARD_DIR) if nickname.lower() in f.lower()]
+        filenames = [f for f in Card.CARD_DIR.iterdir() if nickname.lower() in f.name.lower()]
         if not filenames:
             raise cls.UnknownCard(f'Unknown card nickname or ID "{nickname}".')
         elif len(filenames) == 1:
@@ -324,7 +325,7 @@ class Card:
     def load(cls,
              filepath: Optional[Path] = None,
              identifier: str = None,
-             card_dir: Path = None,
+             card_dir: Optional[Path] = None,
              decoder: Callable = base64.b64decode
              ) -> 'Card':
 
@@ -335,7 +336,7 @@ class Card:
         if not filepath:
             filepath = cls.lookup(identifier=identifier, card_dir=card_dir)
         try:
-            with open(str(filepath), 'rb') as file:
+            with open(filepath, 'rb') as file:
                 card_bytes = decoder(file.read())
         except FileNotFoundError:
             raise cls.UnknownCard
@@ -343,4 +344,4 @@ class Card:
         return instance
 
     def delete(self) -> None:
-        os.remove(str(self.filepath))
+        self.filepath.unlink()
