@@ -14,12 +14,14 @@
  You should have received a copy of the GNU Affero General Public License
  along with nucypher.  If not, see <https://www.gnu.org/licenses/>.
 """
+
 from abc import ABC, abstractmethod
+from difflib import get_close_matches
 from typing import Optional
 
 import requests
+from constant_sorrow.constants import SLOW, MEDIUM, FAST, FASTEST
 from web3 import Web3
-from web3.gas_strategies.rpc import rpc_gas_price_strategy
 from web3.types import Wei, TxParams
 
 
@@ -54,6 +56,13 @@ class EthereumGasPriceDatafeed(Datafeed):
     _speed_names = NotImplemented
     _default_speed = NotImplemented
 
+    _speed_equivalence_classes = {
+        SLOW: ('slow', 'safeLow', 'low'),
+        MEDIUM: ('medium', 'standard', 'average'),
+        FAST: ('fast', 'high'),
+        FASTEST: ('fastest', )
+    }
+
     @abstractmethod
     def _parse_gas_prices(self):
         return NotImplementedError
@@ -61,29 +70,49 @@ class EthereumGasPriceDatafeed(Datafeed):
     def get_gas_price(self, speed: Optional[str] = None) -> Wei:
         speed = speed or self._default_speed
         self._parse_gas_prices()
-        gas_price_wei = Wei(self.gas_prices[speed])
+        gas_price_wei = Wei(self.gas_prices[self.get_canonical_speed(speed)])
         return gas_price_wei
 
     @classmethod
-    def construct_gas_strategy(cls):
+    def construct_gas_strategy(cls, speed: Optional[str] = None):
         def gas_price_strategy(web3: Web3, transaction_params: TxParams = None) -> Wei:
             feed = cls()
-            gas_price = feed.get_gas_price()
+            gas_price = feed.get_gas_price(speed=speed)
             return gas_price
         return gas_price_strategy
 
+    @classmethod
+    def get_canonical_speed(cls, speed: str):
+        for canonical_speed, speed_names in cls._speed_equivalence_classes.items():
+            if speed.lower() in map(str.lower, speed_names):
+                return canonical_speed
+        else:
+            all_speed_names = [name for names in cls._speed_equivalence_classes.values() for name in names]
+            suggestion = get_close_matches(speed, all_speed_names, n=1)
+            if not suggestion:
+                message = f"'{speed}' is not a valid speed name."
+            else:
+                suggestion = suggestion.pop()
+                message = f"'{speed}' is not a valid speed name. Did you mean '{suggestion}'?"
+            raise LookupError(message)
+
 
 class EtherchainGasPriceDatafeed(EthereumGasPriceDatafeed):
     """Gas price datafeed from Etherchain"""
 
     name = "Etherchain datafeed"
     api_url = "https://www.etherchain.org/api/gasPriceOracle"
-    _speed_names = ('safeLow', 'standard', 'fast', 'fastest')
+    _speed_names = {
+        SLOW: 'safeLow',
+        MEDIUM: 'standard',
+        FAST: 'fast',
+        FASTEST: 'fastest'
+    }
     _default_speed = 'fast'
 
     def _parse_gas_prices(self):
         self._probe_feed()
-        self.gas_prices = {k: int(Web3.toWei(v, 'gwei')) for k, v in self._raw_data.items()}
+        self.gas_prices = {self.get_canonical_speed(k): int(Web3.toWei(v, 'gwei')) for k, v in self._raw_data.items()}
 
 
 class UpvestGasPriceDatafeed(EthereumGasPriceDatafeed):
@@ -91,13 +120,37 @@ class UpvestGasPriceDatafeed(EthereumGasPriceDatafeed):
 
     name = "Upvest datafeed"
     api_url = "https://fees.upvest.co/estimate_eth_fees"
-    _speed_names = ('slow', 'medium', 'fast', 'fastest')
+    _speed_names = {
+        SLOW: 'slow',
+        MEDIUM: 'medium',
+        FAST: 'fast',
+        FASTEST: 'fastest'
+    }
     _default_speed = 'fastest'
 
     def _parse_gas_prices(self):
         self._probe_feed()
-        self.gas_prices = {k: int(Web3.toWei(v, 'gwei')) for k, v in self._raw_data['estimates'].items()}
+        self.gas_prices = {self.get_canonical_speed(k): int(Web3.toWei(v, 'gwei'))
+                           for k, v in self._raw_data['estimates'].items()}
 
 
-# TODO: We can implement here other datafeeds, like the ETH/USD (e.g., https://api.coinmarketcap.com/v1/ticker/ethereum/)
-# suggested in a comment in nucypher.blockchain.eth.interfaces.BlockchainInterface#sign_and_broadcast_transaction
+class ZoltuGasPriceDatafeed(EthereumGasPriceDatafeed):
+    """Gas price datafeed from gas-oracle.zoltu.io"""
+
+    name = "gas-oracle.zoltu.io datafeed"
+    api_url = "https://gas-oracle.zoltu.io"
+    _speed_names = {
+        SLOW: 'percentile_40',
+        MEDIUM: 'percentile_75',
+        FAST: 'percentile_95',
+        FASTEST: 'percentile_98'
+    }
+    _default_speed = 'fast'
+
+    def _parse_gas_prices(self):
+        self._probe_feed()
+        self.gas_prices = dict()
+        for canonical_speed_name, zoltu_speed in self._speed_names.items():
+            gwei_price = self._raw_data[zoltu_speed].split(" ")[0]
+            wei_price = int(Web3.toWei(gwei_price, 'gwei'))
+            self.gas_prices[canonical_speed_name] = wei_price
