@@ -14,11 +14,13 @@
  You should have received a copy of the GNU Affero General Public License
  along with nucypher.  If not, see <https://www.gnu.org/licenses/>.
 """
+
+
 from nucypher.blockchain.eth.events import ContractEventsThrottler
-from nucypher.blockchain.eth.utils import estimate_block_number_for_period
 
 try:
     from prometheus_client import Gauge, Enum, Counter, Info, Histogram, Summary
+    from prometheus_client.registry import CollectorRegistry
 except ImportError:
     raise ImportError('"prometheus_client" must be installed - run "pip install nucypher[ursula]" and try again.')
 
@@ -27,16 +29,12 @@ from eth_typing.evm import ChecksumAddress
 
 import nucypher
 from nucypher.blockchain.eth.actors import NucypherTokenActor
-from nucypher.blockchain.eth.agents import ContractAgency, PolicyManagerAgent, StakingEscrowAgent, WorkLockAgent
+from nucypher.blockchain.eth.agents import ContractAgency, PREApplicationAgent, EthereumContractAgent
 from nucypher.blockchain.eth.interfaces import BlockchainInterfaceFactory
 from nucypher.blockchain.eth.registry import BaseContractRegistry
-from nucypher.datastore.queries import get_policy_arrangements, get_work_orders
+from nucypher.datastore.queries import get_reencryption_requests
 
-from prometheus_client.registry import CollectorRegistry
-
-from typing import Dict, Union
-
-ContractAgents = Union[StakingEscrowAgent, WorkLockAgent, PolicyManagerAgent]
+from typing import Dict, Type
 
 
 class MetricsCollector(ABC):
@@ -113,7 +111,6 @@ class UrsulaInfoMetricsCollector(BaseMetricsCollector):
     def _collect_internal(self) -> None:
         # info
         base_payload = {'app_version': nucypher.__version__,
-                        'teacher_version': str(self.ursula.TEACHER_VERSION),
                         'host': str(self.ursula.rest_interface),
                         'domain': self.ursula.domain,
                         'nickname': str(self.ursula.nickname),
@@ -129,30 +126,30 @@ class UrsulaInfoMetricsCollector(BaseMetricsCollector):
         else:
             self.metrics["availability_score_gauge"].set(-1)
 
-        work_orders = get_work_orders(self.ursula.datastore)
-        self.metrics["work_orders_gauge"].set(len(work_orders))
+        # TODO (#2797): for now we leave a terminology discrepancy here, for backward compatibility reasons.
+        # Update "work orders" to "reencryption requests" when possible.
+        reencryption_requests = get_reencryption_requests(self.ursula.datastore)
+        self.metrics["work_orders_gauge"].set(len(reencryption_requests))
 
         if not self.ursula.federated_only:
-            staking_agent = ContractAgency.get_agent(StakingEscrowAgent, registry=self.ursula.registry)
-            locked = staking_agent.get_locked_tokens(staker_address=self.ursula.checksum_address, periods=1)
-            missing_commitments = staking_agent.get_missing_commitments(
-                checksum_address=self.ursula.checksum_address)
-            decentralized_payload = {'provider': str(self.ursula.provider_uri),
-                                     'active_stake': str(locked),
-                                     'missing_commitments': str(missing_commitments)}
+            application_agent = ContractAgency.get_agent(PREApplicationAgent, registry=self.ursula.registry)
+            authorized = application_agent.get_authorized_stake(staking_provider=self.ursula.checksum_address)
+            decentralized_payload = {'provider': str(self.ursula.eth_provider_uri),
+                                     'active_stake': str(authorized)}
             base_payload.update(decentralized_payload)
 
-            policy_arrangements = get_policy_arrangements(self.ursula.datastore)
-            self.metrics["policies_held_gauge"].set(len(policy_arrangements))
+            # TODO: Arrangements are deprecated and Policies are no longer trackable by arrangement storage.
+            # policy_arrangements = get_policy_arrangements(self.ursula.datastore)
+            # self.metrics["policies_held_gauge"].set(len(policy_arrangements))
 
         self.metrics["host_info"].info(base_payload)
 
 
 class BlockchainMetricsCollector(BaseMetricsCollector):
     """Collector for Blockchain specific metrics."""
-    def __init__(self, provider_uri: str):
+    def __init__(self, eth_provider_uri: str):
         super().__init__()
-        self.provider_uri = provider_uri
+        self.eth_provider_uri = eth_provider_uri
 
     def initialize(self, metrics_prefix: str, registry: CollectorRegistry) -> None:
         self.metrics = {
@@ -162,7 +159,7 @@ class BlockchainMetricsCollector(BaseMetricsCollector):
         }
 
     def _collect_internal(self) -> None:
-        blockchain = BlockchainInterfaceFactory.get_or_create_interface(provider_uri=self.provider_uri)
+        blockchain = BlockchainInterfaceFactory.get_or_create_interface(eth_provider_uri=self.eth_provider_uri)
         self.metrics["current_eth_block_number"].set(blockchain.client.block_number)
 
 
@@ -188,9 +185,6 @@ class StakerMetricsCollector(BaseMetricsCollector):
                                         'All tokens that belong to the staker, including '
                                         'locked, unlocked and rewards',
                                         registry=registry),
-            "missing_commitments_gauge": Gauge(f'{metrics_prefix}_missing_commitments',
-                                               'Currently missed commitments',
-                                               registry=registry),
         }
 
     def _collect_internal(self) -> None:
@@ -218,87 +212,49 @@ class StakerMetricsCollector(BaseMetricsCollector):
         self.metrics["unlocked_tokens_gauge"].set(unlocked_tokens)
         self.metrics["owned_tokens_gauge"].set(owned_tokens)
 
-        # missed commitments
-        missing_commitments = staking_agent.get_missing_commitments(checksum_address=self.staker_address)
-        self.metrics["missing_commitments_gauge"].set(missing_commitments)
 
-
-class WorkerMetricsCollector(BaseMetricsCollector):
-    """Collector for Worker specific metrics."""
-    def __init__(self, domain: str, worker_address: ChecksumAddress, contract_registry: BaseContractRegistry):
+class OperatorMetricsCollector(BaseMetricsCollector):
+    """Collector for Operator specific metrics."""
+    def __init__(self, domain: str, operator_address: ChecksumAddress, contract_registry: BaseContractRegistry):
         super().__init__()
         self.domain = domain
-        self.worker_address = worker_address
+        self.operator_address = operator_address
         self.contract_registry = contract_registry
 
     def initialize(self, metrics_prefix: str, registry: CollectorRegistry) -> None:
         self.metrics = {
             "worker_eth_balance_gauge": Gauge(f'{metrics_prefix}_worker_eth_balance',
-                                              'Worker Ethereum balance',
+                                              'Operator Ethereum balance',
                                               registry=registry),
             "worker_token_balance_gauge": Gauge(f'{metrics_prefix}_worker_token_balance',
-                                                'Worker NuNit balance',
+                                                'Operator NuNit balance',
                                                 registry=registry),
         }
 
     def _collect_internal(self) -> None:
         nucypher_worker_token_actor = NucypherTokenActor(registry=self.contract_registry,
                                                          domain=self.domain,
-                                                         checksum_address=self.worker_address)
+                                                         checksum_address=self.operator_address)
         self.metrics["worker_eth_balance_gauge"].set(nucypher_worker_token_actor.eth_balance)
         self.metrics["worker_token_balance_gauge"].set(int(nucypher_worker_token_actor.token_balance))
 
 
-class WorkLockMetricsCollector(BaseMetricsCollector):
-    """Collector for WorkLock specific metrics."""
-    def __init__(self, staker_address: ChecksumAddress, contract_registry: BaseContractRegistry):
-        super().__init__()
-        self.staker_address = staker_address
-        self.contract_registry = contract_registry
-
-    def initialize(self, metrics_prefix: str, registry: CollectorRegistry) -> None:
-        self.metrics = {
-            "available_refund_gauge": Gauge(f'{metrics_prefix}_available_refund',
-                                            'Available refund',
-                                            registry=registry),
-            "worklock_remaining_work_gauge": Gauge(f'{metrics_prefix}_worklock_refund_remaining_work',
-                                                   'Worklock remaining work',
-                                                   registry=registry),
-            "worklock_refund_completed_work_gauge": Gauge(f'{metrics_prefix}_worklock_refund_completedWork',
-                                                          'Worklock completed work',
-                                                          registry=registry),
-        }
-
-    def _collect_internal(self) -> None:
-        staking_agent = ContractAgency.get_agent(StakingEscrowAgent, registry=self.contract_registry)
-        worklock_agent = ContractAgency.get_agent(WorkLockAgent, registry=self.contract_registry)
-
-        self.metrics["available_refund_gauge"].set(
-            worklock_agent.get_available_refund(checksum_address=self.staker_address))
-
-        self.metrics["worklock_remaining_work_gauge"].set(
-            worklock_agent.get_remaining_work(checksum_address=self.staker_address)
-        )
-
-        self.metrics["worklock_refund_completed_work_gauge"].set(
-            staking_agent.get_completed_work(bidder_address=self.staker_address) -
-            worklock_agent.get_refunded_work(checksum_address=self.staker_address)
-        )
-
-
 class EventMetricsCollector(BaseMetricsCollector):
     """General collector for emitted events."""
     def __init__(self,
                  event_name: str,
                  event_args_config: Dict[str, tuple],
                  argument_filters: Dict[str, str],
-                 contract_agent: ContractAgents):
+                 contract_agent_class: Type[EthereumContractAgent],
+                 contract_registry: BaseContractRegistry):
         super().__init__()
         self.event_name = event_name
-        self.contract_agent = contract_agent
+        self.contract_agent_class = contract_agent_class
+        self.contract_registry = contract_registry
 
+        contract_agent = ContractAgency.get_agent(self.contract_agent_class, registry=self.contract_registry)
         # this way we don't have to deal with 'latest' at all
-        self.filter_current_from_block = self.contract_agent.blockchain.client.block_number
+        self.filter_current_from_block = contract_agent.blockchain.client.block_number
         self.filter_arguments = argument_filters
         self.event_args_config = event_args_config
 
@@ -310,14 +266,19 @@ class EventMetricsCollector(BaseMetricsCollector):
             self.metrics[metric_key] = metric_class(metric_name, metric_doc, registry=registry)
 
     def _collect_internal(self) -> None:
+        contract_agent = ContractAgency.get_agent(self.contract_agent_class, registry=self.contract_registry)
         from_block = self.filter_current_from_block
-        to_block = self.contract_agent.blockchain.client.block_number
+        to_block = contract_agent.blockchain.client.block_number
         if from_block >= to_block:
             # we've already checked the latest block and waiting for a new block
             # nothing to see here
             return
 
-        events_throttler = ContractEventsThrottler(agent=self.contract_agent,
+        # update last block checked for the next round - from/to block range is inclusive
+        # increment before potentially long running execution to improve concurrency handling
+        self.filter_current_from_block = to_block + 1
+
+        events_throttler = ContractEventsThrottler(agent=contract_agent,
                                                    event_name=self.event_name,
                                                    from_block=from_block,
                                                    to_block=to_block,
@@ -325,9 +286,6 @@ class EventMetricsCollector(BaseMetricsCollector):
         for event_record in events_throttler:
             self._event_occurred(event_record.raw_event)
 
-        # update last block checked for the next round - from/to block range is inclusive
-        self.filter_current_from_block = to_block + 1
-
     def _event_occurred(self, event) -> None:
         for arg_name in self.event_args_config:
             metric_key = self._get_arg_metric_key(arg_name)
@@ -349,35 +307,6 @@ class CommitmentMadeEventMetricsCollector(EventMetricsCollector):
     def initialize(self, metrics_prefix: str, registry: CollectorRegistry) -> None:
         super().initialize(metrics_prefix=metrics_prefix, registry=registry)
 
-        missing_commitments = self.contract_agent.get_missing_commitments(checksum_address=self.staker_address)
-        if missing_commitments == 0:
-            # has either already committed to this period or the next period
-
-            # use local event filter for initial data
-            last_committed_period = self.contract_agent.get_last_committed_period(staker_address=self.staker_address)
-            arg_filters = {'staker': self.staker_address, 'period': last_committed_period}
-            latest_block = self.contract_agent.blockchain.client.block_number
-            previous_period = self.contract_agent.get_current_period() - 1  # just in case
-            # we estimate the block number for the previous period to start search from since either
-            # - commitment made during previous period for current period, OR
-            # - commitment made during current period for next period
-            block_number_for_previous_period = estimate_block_number_for_period(
-                period=previous_period,
-                seconds_per_period=self.contract_agent.staking_parameters()[1],
-                latest_block=latest_block)
-
-            events_throttler = ContractEventsThrottler(agent=self.contract_agent,
-                                                       event_name=self.event_name,
-                                                       from_block=block_number_for_previous_period,
-                                                       to_block=latest_block,
-                                                       **arg_filters)
-            for event_record in events_throttler:
-                self._event_occurred(event_record.raw_event)
-
-            # update last block checked since we just looked for this event up to and including latest block
-            # block range is inclusive, hence the increment
-            self.filter_current_from_block = latest_block + 1
-
 
 class ReStakeEventMetricsCollector(EventMetricsCollector):
     """Collector for RestakeSet event."""
@@ -387,9 +316,9 @@ class ReStakeEventMetricsCollector(EventMetricsCollector):
 
     def initialize(self, metrics_prefix: str, registry: CollectorRegistry) -> None:
         super().initialize(metrics_prefix=metrics_prefix, registry=registry)
-
+        contract_agent = ContractAgency.get_agent(self.contract_agent_class, registry=self.contract_registry)
         metric_key = self._get_arg_metric_key("reStake")
-        self.metrics[metric_key].set(self.contract_agent.is_restaking(self.staker_address))
+        self.metrics[metric_key].set(contract_agent.is_restaking(self.staker_address))
 
 
 class WindDownEventMetricsCollector(EventMetricsCollector):
@@ -400,52 +329,36 @@ class WindDownEventMetricsCollector(EventMetricsCollector):
 
     def initialize(self, metrics_prefix: str, registry: CollectorRegistry) -> None:
         super().initialize(metrics_prefix=metrics_prefix, registry=registry)
-
+        contract_agent = ContractAgency.get_agent(self.contract_agent_class, registry=self.contract_registry)
         metric_key = self._get_arg_metric_key("windDown")
-        self.metrics[metric_key].set(self.contract_agent.is_winding_down(self.staker_address))
+        self.metrics[metric_key].set(contract_agent.is_winding_down(self.staker_address))
 
 
-class WorkerBondedEventMetricsCollector(EventMetricsCollector):
-    """Collector for WorkerBonded event."""
+class OperatorBondedEventMetricsCollector(EventMetricsCollector):
+    """Collector for OperatorBonded event."""
     def __init__(self,
                  staker_address: ChecksumAddress,
-                 worker_address: ChecksumAddress,
-                 event_name: str = 'WorkerBonded',
+                 operator_address: ChecksumAddress,
+                 event_name: str = 'OperatorBonded',
                  *args,
                  **kwargs):
         super().__init__(event_name=event_name, argument_filters={'staker': staker_address}, *args, **kwargs)
         self.staker_address = staker_address
-        self.worker_address = worker_address
+        self.operator_address = operator_address
 
     def initialize(self, metrics_prefix: str, registry: CollectorRegistry) -> None:
         super().initialize(metrics_prefix=metrics_prefix, registry=registry)
+        contract_agent = ContractAgency.get_agent(self.contract_agent_class, registry=self.contract_registry)
         self.metrics["current_worker_is_me_gauge"] = Gauge(f'{metrics_prefix}_current_worker_is_me',
                                                            'Current worker is me',
                                                            registry=registry)
 
         # set initial value
         self.metrics["current_worker_is_me_gauge"].set(
-            self.contract_agent.get_worker_from_staker(self.staker_address) == self.worker_address)
+            contract_agent.get_worker_from_staker(self.staker_address) == self.operator_address)
 
     def _event_occurred(self, event) -> None:
         super()._event_occurred(event)
+        contract_agent = ContractAgency.get_agent(self.contract_agent_class, registry=self.contract_registry)
         self.metrics["current_worker_is_me_gauge"].set(
-            self.contract_agent.get_worker_from_staker(self.staker_address) == self.worker_address)
-
-
-class WorkLockRefundEventMetricsCollector(EventMetricsCollector):
-    """Collector for WorkLock Refund event."""
-
-    def __init__(self, staker_address: ChecksumAddress, event_name: str = 'Refund', *args, **kwargs):
-        super().__init__(event_name=event_name, argument_filters={'sender': staker_address}, *args, **kwargs)
-        self.staker_address = staker_address
-
-    def initialize(self, metrics_prefix: str, registry: CollectorRegistry) -> None:
-        super().initialize(metrics_prefix=metrics_prefix, registry=registry)
-        self.metrics["worklock_deposited_eth_gauge"] = Gauge(f'{metrics_prefix}_worklock_current_deposited_eth',
-                                                             'Worklock deposited ETH',
-                                                             registry=registry)
-
-    def _event_occurred(self, event) -> None:
-        super()._event_occurred(event)
-        self.metrics["worklock_deposited_eth_gauge"].set(self.contract_agent.get_deposited_eth(self.staker_address))
+            contract_agent.get_worker_from_staker(self.staker_address) == self.operator_address)
